#include "stdio.h"
#include "time.h"
#include "stdlib.h"
#include "string.h"
#include "math.h"
#include "nrutil.h"//**from utility file nrutil.c, from the Numerical Recipes
#include "nrutil.c"//**from utility file nrutil.c, from the Numerical Recipes
#include "nr.h"//////**from utility file nrutil.c, from the Numerical Recipes
#include "fitsio.h"//** for readin fits file, from fitsio package  
//** modified from the original codes in the Numerical Recipes 
#include "funcs2D.c"
#include "mrqmin2D.c"
#include "mrqcof2D.c"
#include "mrqmin2D1.c"
#include "mrqcof2D1.c"
#include "mrqmin2D2.c"
#include "mrqcof2D2.c"
#include "gaussj.c"
#include "covsrt2D.c"
#include "gammln.c"

void main(int argc, char *argv[])
{
  //////////////////////////print tips///////////////////////////////
  while(argc<4)
    {
      printf("\n Please write cluster parameters to data.inf in required format:\n");
      printf("\n Object_name obsID RA Dec z pixle/30kpc  No.Xphoto cent_X  cent_Y bgX  bgY flag\n");
      printf("\n and input the object data file & exposure file with options:\n");
      printf(" -d objectname.fits -x exposurename.fits\n");
      printf("\n Then you can get dynamical parameters with:\n");
      printf(" -c: the concentration index,\n");
      printf(" -w: the centroid shift,\n");
      printf(" -p: the power ratio,\n");
      printf(" -u: the uniformity index,\n");
      printf(" -a: the asymmetry factor,\n");
      printf(" -r: the residual distribution factor,\n");
      printf("\n and obtain images with -plot for:");
      printf(" observation data in obj_obs.eps; \n ");
      printf("    the model plot in obj_model.eps ");
      printf(" & the residual images in obj_residual.eps.\n");
      exit(0);
    }
  /////////////////////judge the input commands//////////////////////
  int i=1,flagc=0,flagw=0,flagp=0,flagu=0,flaga=0,flagr=0,flagplot=0,flagd=0,flagx=0;
  char objname[100],expname[100];
  while(i<argc)
    {
      if(strcmp(argv[i],"-c")==0)
	{
	  flagc=1;
	}
      if(strcmp(argv[i],"-w")==0)
	{
	  flagw=1;
	}
      if(strcmp(argv[i],"-p")==0)
 	{
	  flagp=1;
	}
      if(strcmp(argv[i],"-u")==0)
	{
	  flagu=1;
	}
      if(strcmp(argv[i],"-a")==0)
	{
	  flaga=1;
	}
      if(strcmp(argv[i],"-r")==0)
 	{
	  flagr=1;
	}
      if(strcmp(argv[i],"-d")==0)
 	{
	  flagd=1;
	  strcpy(objname,argv[i+1]);
	}
      if(strcmp(argv[i],"-x")==0)
 	{
	  flagx=1;
	  strcpy(expname,argv[i+1]);
	}
      i++;
    }

  if(flagd==0 || flagx==0)
    {
      printf("\n Please input the object & exposure maps!\n");
      exit(0);
    }

  ///////////////get key parameters from fits header/////////////////
  int clstrID[1],expID[1],nrows[1],ncols[1],enrows[1],encols[1]; long int Nt;
  char file[100];
  float pntx[1],pnty[1],epntx[1],epnty[1],offx[1],offy[1],eoffx[1],eoffy[1];
  //get object image ObsID
  strcpy(file,objname);
  void readkeypar(char fitsname[], int *n, int *nrws, int *ncls, float *pntx, float *pnty, float *offx, float *offy);
  readkeypar(file,clstrID,nrows,ncols,pntx,pnty,offx,offy);
  Nt=*nrows*(*ncols);
  //get exposure map ObsID
  strcpy(file,expname);
  readkeypar(file,expID,enrows,encols,epntx,epnty,eoffx,eoffy);
  if(*expID!=*clstrID)
    {
      printf("\nObject: ObsID=%d, but exposure image: ObsID=%d!\nExit.\n",*clstrID,*expID);
      exit(0);
    }

  /////////////////////////get cluster parameters//////////////////////
  void readpara(int id, float *intcx, float *intcy, char *name, float *RA, float *Dec, float *rdt, float *bgx, float *bgy, float *radi, float *tgsca, int *flagid);
  int id,flagid[1]; char clstrname[20];
  float intcx[1],intcy[1],RA[1],Dec[1],rdt[1],bgx[1],bgy[1],radi[1],tgsca[1],centra,centdec,scale500;
  float integral(float z);
  *flagid=0;
  readpara(*clstrID,intcx,intcy,clstrname,RA,Dec,rdt,bgx,bgy,radi,tgsca,flagid);

  scale500=integral(*rdt)*1000.*3.14159/(180.*3600.);
  scale500=500./scale500/1.5;
  
  centdec=*pnty+(*intcy-(4098.-*offy)/3.)*1.476/3600.;
  centra=*pntx-(*intcx-(4098.-*offx)/3.)*1.476/(3600.*cos(3.14159*centdec/180.));
  if(centra<0.){centra=centra+360.;}
  if(centra>360.){centra=centra-360.;}
  if(*flagid==0)
    {
      printf("\nCan not find parameters in the input file for ObsID: %d!\nExit.\n",*clstrID);
      exit(0);
    }
  else
    {
      printf("\n ##################################################\n ## Cluster Name: %-16s  ObsID: %5d ##\n ##  RA: %9.5f  Dec: %9.5f z: %6.4f     ##\n ##################################################\n",clstrname,*clstrID,*RA,*Dec,*rdt);
    }

  //////////////////////////read object image////////////////////////
  float *x,*y,*fl,*flux;
  x=vector(1,Nt); y=vector(1,Nt); fl=vector(1,Nt); flux=vector(1,Nt);
  void readfits(char fitsname[], float posx[], float posy[], float flx[], float flux[]);
  strcpy(file,objname);
  readfits(file,x,y,fl,flux);
  for(i=1;i<=Nt;i++)
    {fl[i]=fl[i];}

  //////////////////////////read exposure map////////////////////////
  float *epx,*epy,*ept,*eptim;
  strcpy(file,expname);
  epx=vector(1,Nt); epy=vector(1,Nt); ept=vector(1,Nt); eptim=vector(1,Nt);
  readfits(file,epx,epy,ept,eptim);

  /////////////////calculate the mean exposure time//////////////////
  double sumept=0.,meanept;
  long int m=0;
  for(i=1;i<=Nt;i++)
    {
      if(ept[i]>0)
	{
	  m++;
	  sumept=sumept+ept[i];
	}
    }
  meanept=sumept/m;

  /////////calculate the mean and rms for backgrond region/////////////
  void calmean(int ndata, float posx[], float posy[], float fl[], float *bgx, float *bgy, float ept[], float meanept, float *mean, float *rms);
  float mean[1],rms[1];
  calmean(Nt,x,y,fl,bgx,bgy,ept,meanept,mean,rms);
  
  //////////////////////////decide the map region//////////////////////
  void maprgn(int ndata, float posx[], float posy[], float fl[], float ept[], float meanept, float *intcx, float *intcy, float *radi, float *rdt, float *mean, float *rms, float *mapr, float *minx, float *miny, float *maxx, float *maxy, float *minfl, float *maxfl, float *centx, float *centy);
  float mapr[1],minx[1],miny[1],maxx[1],maxy[1],minfl[1],maxfl[1],centx[1],centy[1];
  maprgn(Nt,x,y,fl,ept,meanept,intcx,intcy,radi,rdt,mean,rms,mapr,minx,miny,maxx,maxy,minfl,maxfl,centx,centy);

  ///////////////////save data in the above region//////////////////////
  void savedata(int ndata, float posx[], float posy[], float fl[], float ept[], float meanept, float *minx, float *maxx, float *miny, float *maxy, int *s3, float posx1[], float posy1[], float fl1[], float tme1[], int *s4, float posx2[], float posy2[], float fl2[]);
  int N1[1],N2[1],lngth=2*((int)(*maxx-*minx))*((int)(*maxy-*miny));
  float *x1,*y1,*fl1,*ept1,*x2,*y2,*fl2;
  x1=vector(1,lngth); y1=vector(1,lngth); 
  fl1=vector(1,lngth); ept1=vector(1,lngth);
  x2=vector(1,lngth); y2=vector(1,lngth); fl2=vector(1,lngth);
  N1[0]=0; N2[0]=0;
  savedata(Nt,x,y,fl,ept,meanept,minx,maxx,miny,maxy,N1,x1,y1,fl1,ept1,N2,x2,y2,fl2);

  //////////////////////////image fitting////////////////////////////////
  void imagefit(int id, int s3, float posx1[], float posy1[], float fl1[], float tme1[], int s4, float posx2[], float posy2[], float fl2[], float *tgsca, float *maxfl, float *minfl, float *mapr, float *intcx, float *intcy, float meantm, float mean1, float rms1, float *cof1, float *cof2, float *cof3, float *cof4, float *cof5, float *cof6, float *cof7, float *cof8, float *xisq, float modfl0[], float rsdfl1[], float tr1[], float *minrsd, float *maxrsd, float *tagmod, float *kp, float *fcentx, float *fcenty);
  float cof1[1],cof2[1],cof3[1],cof4[1],cof5[1],cof6[1],cof7[1],cof8[1],xisq[1],minrsd[1],maxrsd[1],tagmod[1],*modfl1,*rsdfl1,*tr1,kapp[1],fcentx[1],fcenty[1];
  modfl1=vector(1,lngth); rsdfl1=vector(1,lngth); tr1=vector(1,lngth);
  imagefit(*clstrID,*N1,x1,y1,fl1,ept1,*N2,x2,y2,fl2,tgsca,maxfl,minfl,mapr,intcx,intcy,meanept,*mean,*rms,cof1,cof2,cof3,cof4,cof5,cof6,cof7,cof8,xisq,modfl1,rsdfl1,tr1,minrsd,maxrsd,tagmod,kapp,fcentx,fcenty);

  ////////////////////calculate dynamical parameters//////////////////////
  //calculate: c (the concentration index)
  void calculate_c(int flagc, float z, int k, float posx[], float posy[], float fl[], float ept[], float mean1, float meantm, float tpa6, float tpa7, float *cni, float *ercni);
  float cni[1],ercni[1];
  if(flagc==1)
    {calculate_c(flagc,*rdt,Nt,x,y,fl,ept,*mean,meanept,*cof6,*cof7,cni,ercni);}
  else
    {*cni=0.; *ercni=0.;}

  //calculate: w (the centroid shift)
  void calculate_w(int flagw, float z, int k, float posx[], float posy[], float fl[], float ept[], float mean1, float rms1, float meantm, float tpa6, float tpa7, float maxx, float maxy, float maxfl, float *cs, float *ercs);
  float w[1],erw[1];
  if(flagw==1)
    {calculate_w(flagw,*rdt,Nt,x,y,fl,ept,*mean,*rms,meanept,*cof6,*cof7,*centx,*centy,*maxfl,w,erw);}
  else
    {*w=0.; *erw=0.;}

  //calculate: p (the power ratio)
  void calculate_p(int flagp, float z, int k, float posx[], float posy[], float fl[], float ept[], float mean1, float meantm, float tpa6, float tpa7, float *pp, float *erpp);
  float pp[1],erpp[1];
  if(flagp==1)
    {calculate_p(flagp,*rdt,Nt,x,y,fl,ept,*mean,meanept,*cof6,*cof7,pp,erpp);}
  else
    {*pp=0.; *erpp=0.;}
  
  //calculate: a (the asymmetry factor)
  void calculate_a(int flaga, int s3, float posx1[], float posy1[], float fl1[], float tme1[], float modfl0[], float flux[], float tim[], int nrows, float *tagmod, float meantm, float tpa6, float tpa7, float *asym, float *erasym);
  float asym[1],erasym[1],flux_x0,flux_y0;
  flux_x0=*fcentx; flux_y0=*fcenty;
  if(flaga==1)
    {calculate_a(flaga,*N1,x1,y1,fl1,ept1,modfl1,flux,eptim,*nrows,tagmod,meanept,flux_x0,flux_y0,asym,erasym);}
  else
    {*asym=0.; *erasym=0.;}

  //calculate: R (the residual distribution factor)
  void calculate_r(int flagr, int s3, float posx1[], float posy1[], float fl1[], float tme1[], float tr1[], float rsdfl1[], float modfl0[], float mean1, float rms1, float meantm, float *dis, float *erdis);
  float dis[1],erdis[1];
  if(flagr==1)
    {calculate_r(flagr,*N1,x1,y1,fl1,ept1,tr1,rsdfl1,modfl1,*mean,*rms,meanept,dis,erdis);}
  else
    {*dis=0.; *erdis=0.;}
  
  //calculate: u (the uniformity index)
  void calculate_u(int flagu, int s3, float posx1[], float posy1[], float fl1[], float tme1[], float tr1[], float mean1, float rms1, float meantm, float tpa6, float tpa7, float *un, float *erun);
  float un[1],erun[1];
  if(flagu==1)
    {calculate_u(flagu,*N1,x1,y1,fl1,ept1,tr1,*mean,*rms,meanept,*cof6,*cof7,un,erun);}
  else
    {*un=0.; *erun=0.;}

  float tk,ta,dlt,edlt;
  //tk=((int)((*kapp+0.005)*100))/100.;
  //ta=((int)((*asym+0.005)*100))/100.;
  //printf("%f %f\n",tk,ta);
  dlt=0.68*(*asym)+0.73*(*kapp)+0.21;
  edlt=*erasym*0.68;

  ////////////////////////print all parameters////////////////////////////
  printf("\nAll parameters for %s is listed as below:\n",clstrname);
  printf("Name                      ObsID    R.A.      Dec.     z      c     ec    w     ew    p     ep    k     a    ea  dlt  edlt\n");
  printf("%-25s %5d %9.5f %9.5f %6.4f %5.2f %5.2f %5.2f %5.2f %5.2f %5.2f %5.2f %4.2f %4.2f %4.2f %4.2f\n",clstrname,*clstrID,*RA,*Dec,*rdt,*cni,*ercni,*w,*erw,*pp,*erpp,*kapp,*asym,*erasym,dlt,edlt);

  FILE *fp;
  char filename[50];
  sprintf(filename,"%d_parameter.dat",*clstrID);
  fp=fopen(filename,"w+");
  fprintf(fp,"%-25s %5d %9.5f %9.5f %6.4f %5.2f %5.2f %5.2f %5.2f %5.2f %5.2f %6.2f %5.2f %5.2f %5.2f %5.2f %6.3f %6.2f %4.2f %4.2f %5.2f %7.2f %7.2f %9.6f %8.3f\n",clstrname,*clstrID,*RA,*Dec,*rdt,*cni,*ercni,*w,*erw,*pp,*erpp,*asym,*erasym,*kapp,dlt,edlt,*cof1,*cof2,*cof3,*cof4,*cof5,*cof6,*cof7,*cof8,*xisq);
  fclose(fp);

//  float tagmin=0.0001;
//  if(tagmin<*minfl) {tagmin=*minfl;}
//  sprintf(filename,"%d_plot.dat",*clstrID);
//  fp=fopen(filename,"w+");
//  fprintf(fp,"%-25s %f %f %f %f %f %f %f %6.4f %f %f %5.2f %5.2f %5.2f %5.3f %5.2f %5.2f %f %f %f %f %f %f %f %f %f %f %f\n",clstrname,centra,centdec,*intcx,*intcy,*cof4,*cof6,*cof7,*rdt,scale500,*tgsca,*cni,*w,*pp,*dis,*asym,*un,*minx,*maxx,*miny,*maxy,tagmin,*maxfl,*minrsd,*maxrsd,*mean,*rms, *tagmod);
//  fclose(fp);

  //////////////////////////plot cluster images///////////////////////////
  
  //plot model image
  void plotmod(int id);
  if(flagplot==1)
    {
      printf("\n......plotting images......\n");
      plotmod(*clstrID);
    }

  //plot observation image
  void plotobs(int id);
  if(flagplot==1)
    {
      plotobs(*clstrID);
    }
  
  //plot residual image
  void plotrsd(int id);
  if(flagplot==1)
    {
      plotrsd(*clstrID);
    }
}

///////////////////////////////////////////////////////////
////////////////////Read fits header///////////////////////
///////////////////////////////////////////////////////////

void readkeypar(char fitsname[], int *n, int *nrws, int *ncls, float *pntx, float *pnty, float *offx, float *offy)
{
  char CltrID[10];
  int status=0;
  fitsfile *fptr;

  //*** open file  
  fits_open_file(&fptr, fitsname, READONLY, &status);
  //*** read source id
  fits_read_key(fptr,TSTRING,"OBS_ID", CltrID, NULL, &status);
  // printf("OBS_ID: %s\n", CltrID);
  //*** trans id to "int"
  int nid; nid=atoi(CltrID);
  
  //*** get file rows and columns
  int nrows,ncols;
  float offsetx,offsety,pointx,pointy;
  fits_movabs_hdu(fptr, 1, NULL, &status);
  fits_read_key(fptr,TINT,"NAXIS1", &nrows, NULL, &status);
  fits_read_key(fptr,TINT,"NAXIS2", &ncols, NULL, &status);
  fits_read_key(fptr,TFLOAT,"CRVAL1P", &offsetx, NULL, &status);
  fits_read_key(fptr,TFLOAT,"CRVAL2P", &offsety, NULL, &status);
  fits_read_key(fptr,TFLOAT,"CRVAL1", &pointx, NULL, &status);
  fits_read_key(fptr,TFLOAT,"CRVAL2", &pointy, NULL, &status);
  fits_close_file(fptr, &status);
  //printf("%d\n",k);
  *n=nid; *nrws=nrows; *ncls=ncols;
  *pntx=pointx; *pnty=pointy; *offx=offsetx; *offy=offsety;
}

///////////////////////////////////////////////////////////////
/////////////////////////Read fits file////////////////////////
///////////////////////////////////////////////////////////////

void readfits(char fitsname[], float posx[], float posy[], float flx[], float flux[])
{
  char CltrID[10];
  int status=0;
  fitsfile *fptr;

  //*** open file  
  fits_open_file(&fptr, fitsname, READONLY, &status);
  
  //*** get file rows and columns
  int nrows, ncols;
  fits_movabs_hdu(fptr, 1, NULL, &status);
  fits_read_key(fptr,TINT,"NAXIS1", &nrows, NULL, &status);
  fits_read_key(fptr,TINT,"NAXIS2", &ncols, NULL, &status);
  
  // *** read data as 1-D array
  int anynul,i,j,k,*nulval;
  long int nelements;
  long fpixel;
  float min,max;
  nelements=nrows*ncols;

  nulval=ivector(1,1); 

  max=-1000; min=10000; status=0; fpixel=1;
  k=0; nulval[1]=1;
 
  fits_read_img(fptr,TFLOAT,fpixel,nelements,&nulval,flux+1, &anynul, &status);

  for(i=1;i<=nelements;i++)
    {
      if(flux[i]<min) {min=flux[i];}
      if(flux[i]>max) {max=flux[i];}
    }
  //printf("min=%f,max=%f\n",min,max);    

  for(i=1;i<=nrows;i++)
    {
      for(j=1;j<=ncols;j++)
	{
	  k++;
	  posx[k]=i;
	  posy[k]=j;
	  flx[k]=flux[(j-1)*nrows+i];
	}
    }
  fits_close_file(fptr, &status);
  //for(i=1;i<=10;i++) {printf("%f %f %f\n",posx[i],posy[i],flx[i]);}
}

///////////////////////////////////////////////////////////
///////////////////////Read cluster parameters/////////////
///////////////////////////////////////////////////////////

void readpara(int id, float *intcx, float *intcy, char *name, float *RA, float *Dec, float *rdt, float *bgx, float *bgy, float *radi, float *tgsca, int *flagid)
{
  FILE *fp;
  int i,obsid,cnt,phx0,phy0;
  float ra,dec,rds,sca,imx0,imy0,imr;
  char altrname[100],fg[10],cat[20];

  *flagid=0;

  fp=fopen("total.inf","r");
  for(i=1;i<=2000;i++)
    {
      fscanf(fp,"%s %d %f %f %f %f %d %f %f %d %d %s %s",altrname,&obsid,&ra,&dec,&rds,&sca,&cnt,&imx0,&imy0,&phx0,&phy0,cat,fg);
      if(id==obsid)
	{
	  if(strcmp(fg,"2")==0)
	    {printf("\n***Warning: the object image has low S/N, \n            the dynamical parameters may not reliable!\n");}
	  if(strcmp(fg,"3")==0)
	    {printf("\n***Warning: the object exceeds CCD coverage, \n            the dynamical parameters may not reliable!\n");}
	  *flagid=1;
	  *intcx=imx0;
	  *intcy=imy0;
	  strcpy(name,altrname);
	  *RA=ra; *Dec=dec; *rdt=rds;
	  *bgx=1.*phx0; *bgy=1.*phy0;
	  *radi=50.*sca; *tgsca=sca;
	  if(*radi>1000.)
	    *radi=*radi/2.;
	  break;
	}
    }
  fclose(fp);
}

/////////////////////////////////////////////////////////////
///////////////////////Calculate mean and rms////////////////
/////////////////////////////////////////////////////////////
void calmean(int ndata, float posx[], float posy[], float fl[], float *bgx, float *bgy, float ept[], float meanept, float *mean, float *rms)
{
  int i,s0,s1;
  float mean2,*bgfl; bgfl=vector(1,ndata);
  
  s1=0; *mean=0.; *rms=0.; mean2=0.; s0=0;
  for(i=1;i<=ndata;i++)
    {
      if(sqrt(pow(posx[i]-(*bgx),2.)+pow(posy[i]-(*bgy),2.))<50 && ept[i]>=0.1*meanept)
	{
	  s1++;
	  bgfl[s1]=fl[i];
	  *mean=*mean+fl[i];
	}
    }
  *mean=*mean/(1.*s1);

  for(i=1;i<=s1;i++)
    {
      *rms=*rms+pow(bgfl[i]-*mean,2.);
    }
  *rms=sqrt(*rms/(1.*s1));
  //printf("mean:%f rms:%f\n",*mean,*rms);
}

///////////////////////////////////////////////////////////////
//////////////////////////Decide map region////////////////////
///////////////////////////////////////////////////////////////
void maprgn(int ndata, float posx[], float posy[], float fl[], float ept[], float meanept, float *intcx, float *intcy, float *radi, float *rdt, float *mean, float *rms, float *mapr, float *minx, float *miny, float *maxx, float *maxy, float *minfl, float *maxfl, float *centx, float *centy)
{
  int i,flag,maxflag,*no,*no1,*no2;
  float *rad,*rat,radspe,mapsca,rnan;
  
  flag=0; maxflag=0;
  no=ivector(1,2000); no1=ivector(1,2000); no2=ivector(1,2000);
  rad=vector(1,ndata); rat=vector(1,2000);
  for(i=1;i<=2000;i++)
    {
      no[i]=0; no1[i]=0; no2[i]=0; rat[i]=0.; 
    }
  for(i=1;i<=ndata;i++)
    {
      rad[i]=sqrt(pow(posx[i]-*intcx,2)+pow(posy[i]-*intcy,2));
      flag=floor(rad[i]/(*radi/100.))+1;
      no[flag]++;
      if(fl[i]>*mean+3.*(*rms))
	{no1[flag]++;}
      if(ept[i]<0.1*meanept)
	{no2[flag]++;}
      if(maxflag<flag)
	{maxflag=flag;}
    }
  for(i=1;i<=maxflag;i++)
    {
      rat[i]=(1.*no2[i])/(1.*no[i]);
      //printf("%f\n",rat[i]);
      if(rat[i]>0.05)
	{rnan=i*(*radi/100.); break;}
    }

  if(*rdt>5)
    {
      for(i=1;i<=maxflag;i++)
	{
	  rat[i]=(1.*no1[i])/(1.*no[i]);
	  if(rat[i]<0.1)
	    {
	      if(i<=26)
		{
		  radspe=26.7*(*radi/100.);
		}
	      else
		{
		  radspe=i*(*radi/100.);
		}
	      break;
	    }
	  if(i==maxflag)
	    {
	      if(i<=26)
		{
		  radspe=26.7*(*radi/100.);
		}
	      else
		{
		  radspe=rnan;
		}
   	    }
	}
      mapsca=3.6*radspe/60.;
    }
  else
    {
      for(i=1;i<=maxflag;i++)
	{
	  rat[i]=(1.*no1[i])/(1.*no[i]);
	  if(rat[i]<0.1)
	    {
	      if(i<=41)
		{
		  radspe=41.7*(*radi/100.);
		}
	      else
		{
		  radspe=i*(*radi/100.);
		}
	      break;
	    }
	  if(i==maxflag)
	    {
	      if(i<=41)
		{
		  radspe=41.7*(*radi/100.);
		}
	      else
		{
		  radspe=rnan;
		}
	    }
	}
      mapsca=2.4*radspe/(*radi);
    }    
  *mapr=1.2*radspe;
  
  *minx=10000.; *maxx=0.; *miny=10000.; *maxy=0.; *minfl=10000.; *maxfl=0.;
  for(i=1;i<=ndata;i++)
    {
      if(rad[i]<*mapr && ept[i]>=0.1*meanept)
	{
	  if(posx[i]<*minx) {*minx=posx[i];}
	  if(posx[i]>*maxx) {*maxx=posx[i];}
	  if(posy[i]<*miny) {*miny=posy[i];}
	  if(posy[i]>*maxy) {*maxy=posy[i];}
	  if(fl[i]<*minfl) {*minfl=fl[i];}
	  if(fl[i]>*maxfl) {*maxfl=fl[i]; *centx=posx[i]; *centy=posy[i];}
	}
    }
}

//////////////////////////////////////////////////////////////
///////////////////////save data in above region//////////////
//////////////////////////////////////////////////////////////
void savedata(int ndata, float posx[], float posy[], float fl[], float ept[], float meanept, float *minx, float *maxx, float *miny, float *maxy, int *N1, float posx1[], float posy1[], float fl1[], float tme1[], int *N2, float posx2[], float posy2[], float fl2[])
{
  int i,s3=0,s4=0;
  for(i=1;i<=ndata;i++)
    {
      if(posx[i]>=*minx && posx[i]<=*maxx && posy[i]>=*miny && posy[i]<=*maxy)
	{
	  s3++;
	  posx1[s3]=posx[i];
	  posy1[s3]=posy[i];
	  fl1[s3]=fl[i];
	  tme1[s3]=ept[i];
	  if(ept[i]>=0.1*meanept)
	    {
	      s4++;
	      posx2[s4]=posx[i];
	      posy2[s4]=posy[i];
	      fl2[s4]=fl[i];
	    }
	}
    }
  *N1=s3; *N2=s4;
}

////////////////////////////////////////////////////////
/////////////////////////Image fitting//////////////////
////////////////////////////////////////////////////////
void imagefit(int id, int s3, float posx1[], float posy1[], float fl1[], float tme1[], int s4, float posx2[], float posy2[], float fl2[], float *tgsca, float *maxfl, float *minfl, float *mapr, float *intcx, float *intcy, float meantm, float mean1, float rms1, float *cof1, float *cof2, float *cof3, float *cof4, float *cof5, float *cof6, float *cof7, float *cof8, float *xisq, float modfl0[], float rsdfl1[], float tr1[], float *minrsd, float *maxrsd, float *tagmod, float *kp, float *fcentx, float *fcenty)
{
  printf("\n......fitting image......\n");
  FILE *fp2;
  float *x1,*x2,*y,*a,**covar,**alpha,*chisq,*alamda,*aa,*yy,*dyda,t1,t2;
  int i,j,ndata,*ia,ma,na,*tag;
 
  void mrqmin2D1(float x1[], float x2[], float y[], int ndata, float a[], int ia[], int ma, float **covar, float **alpha, float *chisq, void funcs2D(float, float, float [], float *, float [], int), float *alamda);
 
  void mrqmin2D2(float x1[], float x2[], float y[], int ndata, float a[], int ia[], int ma, float **covar, float **alpha, float *chisq, void funcs2D(float, float, float [], float *, float [], int), float *alamda);

  ma=16; na=ma;
  a=vector(1,ma); ia=ivector(1,ma); aa=vector(1,na); dyda=vector(1,na);
  covar=matrix(1,ma,1,ma); alpha=matrix(1,ma,1,ma);
  chisq=vector(1,1); alamda=vector(1,1); yy=vector(1,1);

  a[1]=*maxfl; a[2]=*mapr/12.; a[3]=0.8; a[4]=0.2;// try 0.8
  a[5]=1.; a[6]=*intcx; a[7]=*intcy; a[8]=-0.003;
  //printf("%f %f %f %f %f %f %f %f\n",a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8]);

  ma=8; na=ma;
  for(i=1;i<=ma;i++)
    {ia[i]=1;}
  //ia[6]=0; ia[7]=0;

  float tagchisq1; tagchisq1=100000.;
  int tagloop1; tagloop1=0;

  for(i=1;i<=150;i++)
    {
      tagloop1++;
      *alamda=-5.;
      mrqmin2D2(posx2,posy2,fl2,s4,a,ia,ma,covar,alpha,chisq,funcs2D,alamda);
      if(*chisq<tagchisq1*0.99 || tagloop1<=20)
	{
	  tagchisq1=*chisq;
	  //printf("%f %f %f %f %f %f %f %f\n",a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8]);
	}
      else if(a[3]>0.5 && a[4]>0.05 && a[4]<0.6 && a[8]>0.)
	{
	  //printf("%f %f %f %f %f %f %f %f\n",a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8]);
	  break;
	}
      else
	{tagchisq1=tagchisq1*1.2;}
      if(a[3]<=0.5) {a[3]=0.8;}
      if(a[4]<=0.05) {a[4]=0.2;}
      if(a[4]>=0.6) {a[4]=0.5;}
      //if(a[8]<0.) {a[8]=0.00003;}
    }
  //printf("1 beta model: %d loops, least chisq: %f\n",tagloop1-1,*chisq);
  *xisq=*chisq;

  float tpa1,tpa2,tpa3,tpa4,tpa5,tpa6,tpa7,tpa8;
  tpa1=a[1]; tpa2=a[2]; tpa3=a[3]; tpa4=a[4];
  tpa5=a[5]; tpa6=a[6]; tpa7=a[7]; tpa8=a[8];

  a[1]=*maxfl; a[2]=*mapr/12.; a[3]=1.2; a[4]=0.2; //try 1.2
  a[5]=1.; a[6]=*intcx; a[7]=*intcy; a[8]=-0.003;
  //printf("%f %f %f %f %f %f %f %f\n",a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8]);

  ma=8; na=ma;
  for(i=1;i<=ma;i++)
    {ia[i]=1;}
  //ia[6]=0; ia[7]=0;

  tagchisq1=100000.; tagloop1=0;
    
  for(i=1;i<=150;i++)
    {
      tagloop1++;
      *alamda=-5.;
      mrqmin2D2(posx2,posy2,fl2,s4,a,ia,ma,covar,alpha,chisq,funcs2D,alamda);
      if(*chisq<tagchisq1*0.99 || tagloop1<=20)
	{
	  tagchisq1=*chisq;
	  //printf("%f %f %f %f %f %f %f %f\n",a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8]);
	}
      else if(a[3]>0.5 && a[4]>0.05 && a[4]<0.6 && a[8]>0.)
	{
	  //printf("%f %f %f %f %f %f %f %f\n",a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8]);
	  break;
	}
      else
	{tagchisq1=tagchisq1*1.2;}
      if(a[3]<=0.5) {a[3]=0.8;}
      if(a[4]<=0.05) {a[4]=0.2;}
      if(a[4]>=0.6) {a[4]=0.5;}
      //if(a[8]<0.) {a[8]=0.00003;}
    }
  //printf("1 beta model: %d loops, least chisq: %f\n",tagloop1-1,*chisq);

  if(*xisq>*chisq)
    {
      //      tpa1=a[1];  tpa2=a[2]; 
      tpa3=a[3];  tpa4=a[4];   
      tpa6=a[6];  tpa7=a[7];
    }
  //printf("%f %f\n",tpa3,tpa4);
  //a[3]=((int)(100*(a[3]+0.005)))/100.;
  //a[4]=((int)(100*(a[4]+0.005)))/100.;
  *kp=(1.+tpa4)/tpa3;
  *fcentx=tpa6;
  *fcenty=tpa7;
  //printf("%f %f\n",tpa6,tpa7);
  float tagmin;
  tagmin=*minfl;

//  int bd;
//  bd=(int)(100.*(*tgsca));
//  float fl_tot,fl_90,fl_50,r_90,r_50,phyr_90,phyr_50;
//  fl_tot=0.; fl_90=0.; r_90=0.;
//  for(i=0;i<=bd;i++)
//    {
//      fl_tot=fl_tot+(tpa1*pow(1.+pow(i*1.0/tpa2,2.),-1.*tpa3))*(1.*i);
//    }
//  for(i=0;i<=bd;i++)
//    {
//      fl_90=fl_90+(tpa1*pow(1.+pow(i*1.0/tpa2,2.),-1.*tpa3))*(1.*i);
//      if(fl_90>=0.9*fl_tot)
//	{
//	  r_90=i*1.;
//	  break;
//	}
//    }
//  for(i=0;i<=bd;i++)
//    {
//      fl_50=fl_50+(tpa1*pow(1.+pow(i*1.0/tpa2,2.),-1.*tpa3))*(1.*i);
//      if(fl_50>=0.5*fl_tot)
//	{
//	  r_50=i*1.;
//	  break;
//	}
//    }
//  phyr_90=r_90*(30./(*tgsca))/1000.;
//  phyr_50=r_50*(30./(*tgsca))/1000.;
//  printf("30kpc = %.2f;r90=%.3f Mpc;r50=%.3f Mpc\n",*tgsca,phyr_90,phyr_50);

  float sumfl,summodfl,tagr,tagmodfl,cnt1,cnt2;
  char filename[50];
  sprintf(filename,"%d_obj.dat",id);
  sumfl=0.; tagmodfl=0.; 

  fp2=fopen(filename,"w+");
  fprintf(fp2,"%6.2f %6.2f %8.6f\n",posx1[1],posy1[1],tagmin);
  for(i=2;i<=s3;i++)
    {
      if(posx1[i]-posx1[i-1]>=1)
	{fprintf(fp2,"\n");}
      if(tme1[i]>=0.1*meantm && fl1[i]>=tagmin)
	{
	  fprintf(fp2,"%6.2f %6.2f %8.6f\n",posx1[i],posy1[i],fl1[i]);
	  if(fl1[i]>mean1+3*rms1)
	    {
	      sumfl=sumfl+fl1[i];
	    }
	}
      else
	{fprintf(fp2,"%6.2f %6.2f %8.6f\n",posx1[i],posy1[i],tagmin);}
    }
  fclose(fp2);

  float *tx1,*ty1;
  
  tx1=vector(1,s3); ty1=vector(1,s3);
  sprintf(filename,"%d_mod.dat",id);

  fp2=fopen(filename,"w+");
  fprintf(fp2,"%6.2f %6.2f %8.6f\n",posx1[1],posy1[1],tagmin);
  tx1[1]=(posx1[1]-tpa6)*cos(tpa5)+(posy1[1]-tpa7)*sin(tpa5);
  ty1[1]=(posy1[1]-tpa7)*cos(tpa5)-(posx1[1]-tpa6)*sin(tpa5);
  tr1[1]=sqrt(pow(tx1[1],2.)*pow(1.-tpa4,2.)+pow(ty1[1],2.))/(1.-tpa4);
  for(i=2;i<=s3;i++)
    {
      tx1[i]=(posx1[i]-tpa6)*cos(tpa5)+(posy1[i]-tpa7)*sin(tpa5);
      ty1[i]=(posy1[i]-tpa7)*cos(tpa5)-(posx1[i]-tpa6)*sin(tpa5);
      tr1[i]=sqrt(pow(tx1[i],2.)*pow(1.-tpa4,2.)+pow(ty1[i],2.))/(1.-tpa4);
      if(posx1[i]-posx1[i-1]>=1)
	fprintf(fp2,"\n");
      if(tme1[i]>=0.1*meantm)
	{
	  modfl0[i]=tpa1*pow(1.+pow(tr1[i]/tpa2,2.),-1.*tpa3)+tpa8;
	  fprintf(fp2,"%6.2f %6.2f %8.6f\n",posx1[i],posy1[i],modfl0[i]);
	}
      else
	{
	  modfl0[i]=tagmin;
	  fprintf(fp2,"%6.2f %6.2f %8.6f\n",posx1[i],posy1[i],modfl0[i]);
	}
    }
  fclose(fp2);

  tagr=1;
  for(i=1;i<10000;i++)
    {
      tagr=tagr+i;
      cnt1=0.;
      cnt2=0.;
      for(j=1;j<=s3;j++)
	{
	  if(tr1[j]>tagr-1 && tr1[j]<=tagr && tme1[j]>=0.1*meantm)
	    {
	      cnt1=cnt1+1.;
	      if(fl1[j]>mean1+3.*rms1)
		{cnt2=cnt2+1.;}
	    }
	}
      if(cnt2<0.25*cnt1)
	{
	  tagmodfl=tpa1*pow(1.+pow(tagr/tpa2,2.),-1.*tpa3)+tpa8;
	  break;
	}
    }
  *tagmod=tagmodfl;
  //printf("%f\n",tagmodfl);

  *maxrsd=-1000.; *minrsd=1000.;
  sprintf(filename,"%d_rsd.dat",id);
  
  //  fp2=fopen(filename,"w+");
  for(i=1;i<=s3;i++)
    {
      if(tme1[i]>=0.1*meantm)
	{rsdfl1[i]=fl1[i]-modfl0[i];}
      else
	{rsdfl1[i]=0.;}
      if(*maxrsd<rsdfl1[i])
	{*maxrsd=rsdfl1[i];}
      if(*minrsd>rsdfl1[i])
	{*minrsd=rsdfl1[i];}
      if(i>2)
	{
	  if(posx1[i]-posx1[i-1]>=1)
	    {fprintf(fp2,"\n");}
	}
      fprintf(fp2,"%6.2f %6.2f %8.6f\n",posx1[i],posy1[i],rsdfl1[i]);
    }
  //  fclose(fp2);

  //////////////////////###########################/////////////////////////
  ma=16; na=ma;
  a[1]=*maxfl; a[2]=*mapr/12.; a[3]=0.8; a[4]=0.2;// try 0.8
  a[5]=1.; a[6]=*intcx; a[7]=*intcy; a[8]=-0.003;
  //printf("%f %f %f %f %f %f %f %f\n",a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8]);

  ma=8; na=ma;
  for(i=1;i<=ma;i++)
    {ia[i]=1;}
  //ia[6]=0; ia[7]=0;

  tagchisq1=100000.;
  tagloop1=0;

  for(i=1;i<=150;i++)
    {
      tagloop1++;
      *alamda=-5.;
      mrqmin2D1(posx2,posy2,fl2,s4,a,ia,ma,covar,alpha,chisq,funcs2D,alamda);
      if(*chisq<tagchisq1*0.99 || tagloop1<=20)
	{
	  tagchisq1=*chisq;
	  //printf("%f %f %f %f %f %f %f %f\n",a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8]);
	}
      else if(a[3]>0.5 && a[4]>0.05 && a[4]<0.6 && a[8]>0.)
	{
	  //printf("%f %f %f %f %f %f %f %f\n",a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8]);
	  break;
	}
      else
	{tagchisq1=tagchisq1*1.2;}
      if(a[3]<=0.5) {a[3]=0.8;}
      if(a[4]<=0.05) {a[4]=0.2;}
      if(a[4]>=0.6) {a[4]=0.5;}
      //if(a[8]<0.) {a[8]=0.00003;}
    }
  //printf("1 beta model: %d loops, least chisq: %f\n",tagloop1-1,*chisq);
  *xisq=*chisq;

  tpa1=a[1]; tpa2=a[2]; tpa3=a[3]; tpa4=a[4];
  tpa5=a[5]; tpa6=a[6]; tpa7=a[7]; tpa8=a[8];

  a[1]=*maxfl; a[2]=*mapr/12.; a[3]=1.2; a[4]=0.2; //try 1.2
  a[5]=1.; a[6]=*intcx; a[7]=*intcy; a[8]=-0.003;
  //printf("%f %f %f %f %f %f %f %f\n",a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8]);

  ma=8; na=ma;
  for(i=1;i<=ma;i++)
    {ia[i]=1;}
  //ia[6]=0; ia[7]=0;

  tagchisq1=100000.; tagloop1=0;
    
  for(i=1;i<=150;i++)
    {
      tagloop1++;
      *alamda=-5.;
      mrqmin2D1(posx2,posy2,fl2,s4,a,ia,ma,covar,alpha,chisq,funcs2D,alamda);
      if(*chisq<tagchisq1*0.99 || tagloop1<=20)
	{
	  tagchisq1=*chisq;
	  //printf("%f %f %f %f %f %f %f %f\n",a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8]);
	}
      else if(a[3]>0.5 && a[4]>0.05 && a[4]<0.6 && a[8]>0.)
	{
	  //printf("%f %f %f %f %f %f %f %f\n",a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8]);
	  break;
	}
      else
	{tagchisq1=tagchisq1*1.2;}
      if(a[3]<=0.5) {a[3]=0.8;}
      if(a[4]<=0.05) {a[4]=0.2;}
      if(a[4]>=0.6) {a[4]=0.5;}
      //if(a[8]<0.) {a[8]=0.00003;}
    }
  //printf("1 beta model: %d loops, least chisq: %f\n",tagloop1-1,*chisq);

  if(*xisq>*chisq)
    {
      tpa1=a[1]; tpa2=a[2]; tpa3=a[3];
      tpa4=a[4]; tpa5=a[5]; tpa6=a[6];
      tpa7=a[7]; tpa8=a[8]; *xisq=*chisq;    
    }
  *cof1=tpa1; *cof2=tpa2; *cof3=tpa3; *cof4=tpa4;
  *cof5=tpa5; *cof6=tpa6; *cof7=tpa7; *cof8=tpa8;
  
}

/////////////////////////////////////////////////////////////////
/////////////////calculate dynamical parameters//////////////////
/////////////////////////////////////////////////////////////////

void calculate_c(int flagc, float z, int k, float posx[], float posy[], float fl[], float ept[], float mean1, float meantm, float tpa6, float tpa7, float *cni, float *ercni)
{
  int i,j;
  float r500,sum1,sum2,rad;
  float integral(float z);
  r500=integral(z)*1000.*3.14159/(180.*3600.);
  r500=500./r500/1.5;
  
  /////////////////////calculate the "c" index///////////////////////
  printf("\n......calculating: c (concentration index)......\n");
  if(z>2)
    {
      printf("No redshift available! \"c\" can NOT be calculated!\n");
      *cni=0.; *ercni=0.;
    }
  else
    {
      int annu,*si,annutag;
      float *sumflc,*meanflc,ci;
      si=ivector(1,20); sumflc=vector(1,20); meanflc=vector(1,20);
      sum1=0.; sum2=0.; annutag=0;

      for(i=1;i<=20;i++)
	{
	  sumflc[i]=0.; si[i]=0;
	}
      for(i=1;i<=k;i++)
	{
	  rad=sqrt(pow(posx[i]-tpa6,2.)+pow(posy[i]-tpa7,2.));
	  if(rad<r500 && ept[i]>=0.1*meantm)
	    {
	      annu=floor(rad/(r500/20.))+1;
	      si[annu]++;
	      sumflc[annu]=sumflc[annu]+fl[i]-mean1;
	    }
	}
      for(i=1;i<=20;i++)
	{
	  meanflc[i]=sumflc[i]/(si[i]*1.);
	  if(si[i]==0.)
	    {annutag=1;
	      printf("No data in %dth annu!\n\"c\" can NOT be calculated!\n",i);}
	  //printf("%f %d %f\n",sumflc[i],si[i],meanflc[i]);
	}
      for(i=1;i<=4;i++)
	{
	  sum1=sum1+(i*(r500/20.)-0.5)*meanflc[i];
	}
      for(i=1;i<=20;i++)
	{
	  sum2=sum2+(i*(r500/20.)-0.5)*meanflc[i];
	}
      if(annutag<1)
	{
	ci=log10(sum1/sum2);
      //printf("%f %f %f\n",sum1,sum2,ci);
	}
      else
	{ci=1.00;}
      
      //////////////////error of c///////////////////////////////
      int tagannu;
      float *cndx,sumcndx,erci;
      cndx=vector(1,10); sumcndx=0.; erci=0.; tagannu=0;

      for(j=1;j<=10;j++)
	{
	  sum1=0.; sum2=0.;
	  for(i=1;i<=20;i++)
	    {
	      sumflc[i]=0.; si[i]=0; meanflc[i]=0.;
	    }
	  for(i=j;i<=k;i=i+10)
	    {
	      rad=sqrt(pow(posx[i]-tpa6,2.)+pow(posy[i]-tpa7,2.));
	      if(rad<r500 && ept[i]>=0.1*meantm)
		{
		  annu=floor(rad/(r500/20.))+1;
		  si[annu]++;
		  sumflc[annu]=sumflc[annu]+fl[i]-mean1;
		}
	    }
	  for(i=1;i<=20;i++)
	    {
	      if(si[i]==0)
		{//printf("zero count of %d annu!",i);
		  meanflc[i]=0.;}
	      else
		{meanflc[i]=sumflc[i]/(si[i]*1.);}
	      //printf("%f %d %f\n",sumfl[i],si[i],meanfl[i]);
	    }
	  for(i=1;i<=4;i++)
	    {
	      sum1=sum1+(i*(r500/20.)-0.5)*meanflc[i];
	    }
	  for(i=1;i<=20;i++)
	    {
	      sum2=sum2+(i*(r500/20.)-0.5)*meanflc[i];
	    }
	  cndx[j]=log10(sum1/sum2);
	  //printf("%f\n",cndx[j]);
	  sumcndx=sumcndx+cndx[j];
	}
      sumcndx=sumcndx/10.;
      for(i=1;i<=10;i++)
	{
	  erci=erci+pow(cndx[i]-sumcndx,2.);
	}
      erci=sqrt(erci/10.);
      if(erci<0.01)
	{erci=0.01;}
      if(annutag==1)
	{erci=1.00;}
      *cni=ci; *ercni=erci;
    }
}

void calculate_p(int flagp, float z, int k, float posx[], float posy[], float fl[], float ept[], float mean1, float meantm, float tpa6, float tpa7, float *pp, float *erpp)
{
  int i,j;
  float r500,rad;
  float integral(float z);
  r500=integral(z)*1000.*3.14159/(180.*3600.);
  r500=500./r500/1.5;

  //////////////////////calculate the "P3/P0"////////////////////////
  printf("\n......calculating: p (power ratio)......\n");

  if(z>2)
    {
      printf("No redshift available! \"p\" can NOT be calculated!\n");
      *pp=0.; *erpp=0.;
    }
  else
    {
      float *phi,*phc,*phs,*rr,*coa,*cob,sum1,sum2,sum3,pr,pr0,pwr;
      phi=vector(1,k); phc=vector(1,k); phs=vector(1,k);
      rr=vector(1,k); coa=vector(1,k); cob=vector(1,k);
      sum1=0.; sum2=0.; sum3=0.;
  
      for(i=1;i<=k;i++)
	{
	  rad=sqrt(pow(posx[i]-tpa6,2.)+pow(posy[i]-tpa7,2.));
	  if(rad<r500 && ept[i]>=0.1*meantm)
	    {
	      phi[i]=atan2(posy[i]-tpa7,posx[i]-tpa6);
	      phc[i]=cos(3.*phi[i]);
	      phs[i]=sin(3.*phi[i]);
	      rr[i]=(fl[i]-mean1)*pow(pow(posx[i]-tpa6,2)+pow(posy[i]-tpa7,2),1.5);
	      coa[i]=rr[i]*phc[i];
	      cob[i]=rr[i]*phs[i];
	      sum1=sum1+coa[i];
	      sum2=sum2+cob[i];
	      sum3=sum3+fl[i];
	    }
	}
      pr=(pow(sum1,2)+pow(sum2,2))/(18.*pow(r500,6));
      pr0=pow(sum3*log(r500),2);
      pwr=log10(pr/pr0);

      float *p3p0,sump3p0,erp3p0;
      p3p0=vector(1,10); sump3p0=0.; erp3p0=0.;
  
      for(j=1;j<=10;j++)
	{
	  sum1=0.; sum2=0.; sum3=0.; pr=0.; pr0=0.;
	  for(i=j;i<=k;i=i+10)
	    {
	      rad=sqrt(pow(posx[i]-tpa6,2.)+pow(posy[i]-tpa7,2.));
	      if(rad<r500 && ept[i]>=0.1*meantm)
		{
		  phi[i]=atan2(posy[i]-tpa7,posx[i]-tpa6);
		  phc[i]=cos(3.*phi[i]);
		  phs[i]=sin(3.*phi[i]);
		  rr[i]=(fl[i]-mean1)*pow(pow(posx[i]-tpa6,2)+pow(posy[i]-tpa7,2),1.5);
		  coa[i]=rr[i]*phc[i];
		  cob[i]=rr[i]*phs[i];
		  sum1=sum1+coa[i];
		  sum2=sum2+cob[i];
		  sum3=sum3+fl[i];
		}
	    }
	  pr=(pow(sum1,2)+pow(sum2,2))/(18.*pow(r500,6));
	  pr0=pow(sum3*log(r500),2);
	  p3p0[j]=log10(pr/pr0);
	  //printf("%f\n",p3p0[j]);
	  sump3p0=sump3p0+p3p0[j];
	}
      sump3p0=sump3p0/10.;
      for(i=1;i<=10;i++)
	{
	  erp3p0=erp3p0+pow(p3p0[i]-sump3p0,2.);
	}
      erp3p0=sqrt(erp3p0/10.);
      if(erp3p0<0.01)
	{erp3p0=0.01;}
      *pp=pwr; *erpp=erp3p0;
    }
}

void calculate_w(int flagw, float z, int k, float posx[], float posy[], float fl[], float ept[], float mean1, float rms1, float meantm, float tpa6, float tpa7, float maxx, float maxy, float maxfl, float *cs, float *ercs)
{
  int i,j;
  float r500;
  float integral(float z);
  r500=integral(z)*1000.*3.14159/(180.*3600.);
  r500=500./r500/1.5;
  
  //////////////////////calculate the "w" factor/////////////////////

  printf("\n......calculating: w (centroid shift)......\n");
  if(z>2)
    {
      printf("No redshift available! \"w\" can NOT be calculated!\n");
      *cs=0.; *ercs=0.;
    }
  else
    {
      float *x1,*x2,*y,*a,**covar,**alpha,*chisq,*alamda,*aa,*yy,*dyda,t1,t2;
      int ndata,*ia,ma,na,*tag;
 
      void mrqmin2D2(float x1[], float x2[], float y[], int ndata, float a[], int ia[], int ma, float **covar, float **alpha, float *chisq, void funcs2D(float, float, float [], float *, float [], int), float *alamda);

      ma=8; na=ma;
      a=vector(1,ma); ia=ivector(1,ma); aa=vector(1,na); dyda=vector(1,na);
      covar=matrix(1,ma,1,ma); alpha=matrix(1,ma,1,ma);
      chisq=vector(1,1); alamda=vector(1,1); yy=vector(1,1);
  
      ia[1]=1; ia[2]=1; ia[3]=1; ia[4]=0;
      ia[5]=0; ia[6]=1; ia[7]=1; ia[8]=1;

      float tagchisq1,xxisq,*posx3,*posy3,*fl3,tmpa1,tmpa2,tmpa3,tmpa4,tmpa5,tmpa6,tmpa7,tmpa8,*fitx,*fity,*delta,meandelta,sumdelta,w,*omg,meanomg,sumx,sumy,sigx,sigy;
      int tagloop1,mm,jj,flag,s1; 
      posx3=vector(1,k); posy3=vector(1,k); fl3=vector(1,k);
      fitx=vector(1,20); fity=vector(1,20); delta=vector(1,20);
      omg=vector(1,10);
  
      meandelta=0.; sumdelta=0.; flag=0; meanomg=0.;
      sumx=0.; sumy=0.; sigx=0.; sigy=0.;
  
      for(mm=1;mm<=20;mm++)
	{
	  s1=0; 
	  for(i=1;i<=k;i++)
	    {
	      if(sqrt(pow(posx[i]-maxx,2.)+pow(posy[i]-maxy,2.))<mm*r500/20. && ept[i]>=0.1*meantm)
		{
		  s1++;
		  posx3[s1]=posx[i]; posy3[s1]=posy[i]; fl3[s1]=fl[i];
		}
	    }
	  //printf("%d\n",s1);
	  a[1]=maxfl; a[2]=mm*r500/10.; a[3]=0.8; a[4]=0.;// try 0.8
	  a[5]=0.; a[6]=maxx; a[7]=maxy; a[8]=0.;
	  //printf("%f %f %f %f %f %f %f %f\n",a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8]);
	  if(s1>=20)
	    {
	      tagchisq1=100000.; tagloop1=0;
	      for(i=1;i<=150;i++)
		{
		  tagloop1++;
		  *alamda=-5.;
		  mrqmin2D2(posx3,posy3,fl3,s1,a,ia,ma,covar,alpha,chisq,funcs2D,alamda);
		  if(*chisq<tagchisq1*0.99 || tagloop1<=20)
		    {
		      tagchisq1=*chisq;
		      //printf("%f %f %f %f %f %f %f %f\n",a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8]);
		    }
		  else if(a[3]>0.5 && a[8]>0.)
		    {
		      //printf("%f %f %f %f %f %f %f %f\n",a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8]);
		      break;
		    }
		  else
		    {tagchisq1=tagchisq1*1.2;}
		  if(a[3]<=0.5) {a[3]=0.8;}
		  //if(a[8]<0.) {a[8]=0.00003;}
		}
	      //printf("1 beta model: %d loops, least chisq: %f\n",tagloop1-1,*chisq);
	      xxisq=*chisq;

	      tmpa1=a[1]; tmpa2=a[2]; tmpa3=a[3]; tmpa4=a[4];
	      tmpa5=a[5]; tmpa6=a[6]; tmpa7=a[7]; tmpa8=a[8];

	      a[1]=maxfl; a[2]=mm*r500/10.; a[3]=1.2; a[4]=0.; //try 1.2
	      a[5]=0.; a[6]=maxx; a[7]=maxy; a[8]=0.;
	      //printf("%f %f %f %f %f %f %f %f\n",a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8]);
	      tagchisq1=100000.; tagloop1=0;
    
	      for(i=1;i<=150;i++)
		{
		  tagloop1++;
		  *alamda=-5.;
		  mrqmin2D2(posx3,posy3,fl3,s1,a,ia,ma,covar,alpha,chisq,funcs2D,alamda);
		  if(*chisq<tagchisq1*0.99 || tagloop1<=20)
		    {
		      tagchisq1=*chisq;
		      //printf("%f %f %f %f %f %f %f %f\n",a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8]);
		    }
		  else if(a[3]>0.5 && a[8]>0.)
		    {
		      //printf("%f %f %f %f %f %f %f %f\n",a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8]);
		      break;
		    }
		  else
		    {tagchisq1=tagchisq1*1.2;}
		  if(a[3]<=0.5) {a[3]=0.8;}
		  //if(a[8]<0.) {a[8]=0.00003;}
		}
	      //printf("1 beta model: %d loops, least chisq: %f\n",tagloop1-1,*chisq);

	      if(xxisq>*chisq)
		{fitx[mm]=a[6]; fity[mm]=a[7];}
	      else
		{fitx[mm]=tmpa6; fity[mm]=tmpa7; a[3]=tmpa3; a[4]=tmpa4;}
	    }
	  else
	    {fitx[mm]=maxx; fity[mm]=maxy;}
	  //printf("%d %f %f %f %f\n",s1,fitx[mm],fity[mm],maxx,maxy);
	  sumx=sumx+fitx[mm]; sumy=sumy+fity[mm];
	}
      sumx=sumx/20.; sumy=sumy/20.;
      for(mm=1;mm<=20;mm++)
	{
	  sigx=sigx+pow(fitx[mm]-sumx,2.);
	  sigy=sigy+pow(fity[mm]-sumy,2.);
	}
      sigx=sqrt(sigx/20.); sigy=sqrt(sigy/20.);
      if(sigx<0.5) {sigx=0.5;}
      if(sigy<0.5) {sigy=0.5;}
      //printf("%f %f\n",sigx,sigy);
      for(mm=1;mm<=20;mm++)
	{
	  if(fabs(fitx[mm]-maxx)<5.*sigx && fabs(fity[mm]-maxy)<5.*sigy)
	    {
	      flag++;
	      delta[flag]=sqrt(pow(fitx[mm]-maxx,2.)+pow(fity[mm]-maxy,2.));
	      meandelta=meandelta+delta[flag];
	    }
	  else if(fabs(fitx[mm]-maxx)<1.5*fabs(maxx-tpa6) && fabs(fity[mm]-maxy)<1.5*fabs(maxy-tpa7))
	    {
	      flag++;
	      delta[flag]=sqrt(pow(fitx[mm]-maxx,2.)+pow(fity[mm]-maxy,2.));
	      meandelta=meandelta+delta[flag];
	    }
	}
      //printf("\n%f %f\n%f %f\n",x0,y0,maxx,maxy);
      meandelta=meandelta/20.;
      for(i=1;i<=flag;i++)
	{
	  sumdelta=sumdelta+pow(delta[i]-meandelta,2.);
	}
      w=log10(sqrt(sumdelta/(flag-1.))/r500);
      //printf("%d %f\n",flag,w);
  
      ////////////////////the error of w/////////////////////////////////
      for(jj=1;jj<=10;jj++)
	{
	  meandelta=0.; sumdelta=0.; flag=0;
	  for(mm=1;mm<=20;mm++)
	    {
	      s1=0; 
	      for(i=jj;i<=k;i=i+10)
		{
		  if(sqrt(pow(posx[i]-maxx,2.)+pow(posy[i]-maxy,2.))<mm*r500/20. && ept[i]>=0.1*meantm)
		    {
		      s1++;
		      posx3[s1]=posx[i]; posy3[s1]=posy[i]; fl3[s1]=fl[i];
		    }
		}
	      a[1]=maxfl; a[2]=mm*r500/10.; a[3]=0.8; a[4]=0.;// try 0.8
	      a[5]=0.; a[6]=maxx; a[7]=maxy; a[8]=0.;
	      //printf("%d\n",s1);
	      if(s1>=20)
		{
		  tagchisq1=100000.; tagloop1=0;
		  for(i=1;i<=150;i++)
		    {
		      tagloop1++;
		      *alamda=-5.;
		      mrqmin2D2(posx3,posy3,fl3,s1,a,ia,ma,covar,alpha,chisq,funcs2D,alamda);
		      if(*chisq<tagchisq1*0.99 || tagloop1<=20)
			{
			  tagchisq1=*chisq;
			}
		      else if(a[3]>0.5 && a[8]>0.)
			{
			  break;
			}
		      else
			{tagchisq1=tagchisq1*1.2;}
		      if(a[3]<=0.5) {a[3]=0.8;}
		      //if(a[8]<0.) {a[8]=0.00003;}
		    }
		  xxisq=*chisq;

		  tmpa1=a[1]; tmpa2=a[2]; tmpa3=a[3]; tmpa4=a[4];
		  tmpa5=a[5]; tmpa6=a[6]; tmpa7=a[7]; tmpa8=a[8];

		  a[1]=maxfl; a[2]=mm*r500/10.; a[3]=1.2; a[4]=0.; //try 1.2
		  a[5]=0.; a[6]=maxx; a[7]=maxy; a[8]=0.;
		  tagchisq1=100000.; tagloop1=0;
    
		  for(i=1;i<=150;i++)
		    {
		      tagloop1++;
		      *alamda=-5.;
		      mrqmin2D2(posx3,posy3,fl3,s1,a,ia,ma,covar,alpha,chisq,funcs2D,alamda);
		      if(*chisq<tagchisq1*0.99 || tagloop1<=20)
			{
			  tagchisq1=*chisq;
			}
		      else if(a[3]>0.5 && a[8]>0.)
			{
			  break;
			}
		      else
			{tagchisq1=tagchisq1*1.2;}
		      if(a[3]<=0.5) {a[3]=0.8;}
		      //if(a[8]<0.) {a[8]=0.00003;}
		    }

		  if(xxisq>*chisq)
		    {fitx[mm]=a[6]; fity[mm]=a[7];}
		  else
		    {fitx[mm]=tmpa6; fity[mm]=tmpa7;}
		}
	      else
		{fitx[mm]=maxx; fity[mm]=maxy;}
	      //printf("%d %f %f\n",s1,fitx[mm],fity[mm]);
	      if(fabs(fitx[mm]-maxx)<5.*sigx && fabs(fity[mm]-maxy)<5.*sigy)
		{
		  flag++;
		  delta[flag]=sqrt(pow(fitx[mm]-maxx,2.)+pow(fity[mm]-maxy,2.));
		  meandelta=meandelta+delta[flag];
		}
	      else if(fabs(fitx[mm]-maxx)<1.5*fabs(maxx-tpa6) && fabs(fity[mm]-maxy)<1.5*fabs(maxy-tpa7))
		{
		  flag++;
		  delta[flag]=sqrt(pow(fitx[mm]-maxx,2.)+pow(fity[mm]-maxy,2.));
		  meandelta=meandelta+delta[flag];
		}
	    }  
	  meandelta=meandelta/(1.*flag);
	  //printf("\n%f %f\n%f %f\n",x0,y0,maxx,maxy);
	  for(i=1;i<=flag;i++)
	    {
	      sumdelta=sumdelta+pow(delta[i]-meandelta,2.);
	    }
	  omg[jj]=log10(sqrt(sumdelta/(flag-1.))/r500);
	  //printf("%d %f\n",flag,omg[jj]);
	  meanomg=meanomg+omg[jj];
	}
      meanomg=meanomg/10.;
  
      float erw; erw=0.;
      for(i=1;i<=10;i++)
	{
	  erw=erw+pow(omg[i]-meanomg,2.);
	}
      erw=sqrt(erw/10.);
      if(erw<0.01)
	{erw=0.01;}
      *cs=w; *ercs=erw;
    }
}

//////////////////////calculate the "R" factor/////////////////////

void calculate_r(int flagr, int s3, float posx1[], float posy1[], float fl1[], float tme1[], float tr1[], float rsdfl1[], float modfl0[], float mean1, float rms1, float meantm, float *dis, float *erdis)
{
  printf("\n......calculating: R (residual distribution factor)......\n");
  int i,j;
  float wdis,sumwd,sumrsd,sumr,tags1; 
  sumwd=0.; sumrsd=0.; sumr=0.; tags1=0.;
  for(i=1;i<=s3;i++)
    {
      if(tme1[i]>0.1*meantm && modfl0[i]>mean1+3.*rms1)
	{
	  tags1=tags1+1.;
	  sumwd=sumwd+fabs(rsdfl1[i]/modfl0[i]);
	  sumrsd=sumrsd+fabs(rsdfl1[i]);
	  sumr=sumr+fabs(modfl0[i]);
	}
    }
  wdis=(sumwd*sumrsd)/(tags1*sumr);
  //printf("%f %f %f %f\n",sumwd,sumrsd,wdis,sumr);
 
  ///////////////////////the error of R/////////////////////////////
  float *dis1,sumdis1,erdis1,ndt;
  dis1=vector(1,s3); sumdis1=0.; erdis1=0.; ndt=0.;
  printf("%f\n",mean1);
  for(j=1;j<=10;j++)
    {
      tags1=0.; sumwd=0.; sumr=0.; sumrsd=0.;
      for(i=j;i<=s3;i=i+10)
	{
	  if(tme1[i]>0.1*meantm && fabs(rsdfl1[i])>3.*rms1 && modfl0[i]>mean1+3.*rms1)
	    {
	      tags1=tags1+1.;
	      sumwd=sumwd+fabs(rsdfl1[i]*rsdfl1[i]*tr1[i]);
	      sumrsd=sumrsd+fabs(rsdfl1[i]*rsdfl1[i]);
	      sumr=sumr+tr1[i];
	    }
	}
      printf("%f %f %f %f %f\n",tags1,sumwd,sumrsd,sumr,ndt);
      if(tags1<1)
	{ndt=ndt+1.;}
      else
	{
	  dis1[j]=sumwd*tags1/(sumrsd*sumr);
	  sumdis1=sumdis1+dis1[j];
	}
    }
  sumdis1=sumdis1/(10.-ndt);
  for(i=1;i<=10;i++)
    {
      erdis1=erdis1+pow(dis1[i]-sumdis1,2.);
    }
  erdis1=sqrt(erdis1/10.);
  if(erdis1<0.001)
    {erdis1=0.001;}
  //printf("%f %f\n",wdis1,erdis1);
  *dis=wdis; *erdis=erdis1;
  if(ndt>=10.)
    {*dis=1.; *erdis=0.001;}
}

void calculate_a(int flaga, int s3, float posx1[], float posy1[], float fl1[], float tme1[], float modfl0[], float flux[], float tim[], int nrows, float *tagmod, float meantm, float tpa6, float tpa7, float *asym, float *erasym)
{
  //////////////////////calculate the "a" factor//////////////////
  printf("\n......calculating: a (asymmetry factor)......\n");
  float asy,sumM;
  int i,j,tags,tagx,tagy,tagi;
  asy=0.; sumM=0.; tags=0;
  for(i=1;i<=s3;i++)
    {
      if(modfl0[i]>*tagmod && tme1[i]>=0.1*meantm)
	{
	  tagx=2*floor(tpa6)-(int)posx1[i];
	  tagy=2*floor(tpa7)-(int)posy1[i];
	  tagi=(tagy-1)*nrows+tagx;
	  if(tim[tagi]>=0.1*meantm)
	    {
	      tags++;
	      asy=asy+pow(fl1[i]-flux[tagi],2.);
	      sumM=sumM+fl1[i]*fl1[i];
	    }
	}
    }
  //printf("%f %f\n",sumM,asy);
  if(sumM>0.)
    {asy=log10(asy/sumM);}
  else
    {asy=0.;}
  //printf("%f\n",asy);
  *asym=asy;
  ////////////////////////error of a////////////////////////////
  float *asy1,sumasy,sumasy1,erasy,ndt1;
  asy1=vector(1,10); sumasy1=0.; erasy=erasy=0.; ndt1=0.;
  //for(i=1;i<=100;i++)
  //{printf("%f\n",fl1[i]);}
  
  for(j=1;j<=10;j++)
    {
      tags=0; sumasy=0.; sumM=0.;
      for(i=j;i<=s3;i=i+10)
	{
	  if(modfl0[i]>*tagmod && tme1[i]>=0.1*meantm)
	    {
	      tagx=2*floor(tpa6)-posx1[i];
	      tagy=2*floor(tpa7)-posy1[i];
	      tagi=(tagy-1)*nrows+tagx;
	      if(tim[tagi]>=0.1*meantm)
		{
		  tags++;
		  sumasy=sumasy+pow(fl1[i]-flux[tagi],2.);
		  sumM=sumM+fl1[i]*fl1[i];
		}
	    }
	}
      //printf("%f %f\n",sumasy,sumM);
      if(sumM==0.)
	{ndt1=ndt1+1.;}
      else
	{
	  asy1[j]=sumasy*100./sumM;
	  sumasy1=sumasy1+asy1[j];
	}
    }
  sumasy1=sumasy1/(10.-ndt1);
  for(i=1;i<=10;i++)
    {
      erasy=erasy+pow(asy1[i]-sumasy1,2.);
    }
  erasy=sqrt(erasy/10.);
  erasy=erasy/(2.3*asy);
  if(erasy<0.01)
    {erasy=0.01;}
  //printf("%f %f %f\n",asy,erasy,ndt1);
  *erasym=erasy;
  if(ndt1==10.)
    {*erasym=0.;}
}
  
void calculate_u(int flagu, int s3, float posx1[], float posy1[], float fl1[], float tme1[], float tr1[], float mean1, float rms1, float meantm, float tpa6, float tpa7, float *un, float *erun)
{
///////////////calculate the "u" index///////////////////
  printf("\n......calculating: u (uniformity index)......\n");
  float unf,sumfl;
  int i,j,flags2;
  sumfl=0.; flags2=0; unf=0.;
  for(i=1;i<=s3;i++)
    {
      if(fl1[i]>mean1+3.*rms1 && tme1[i]>=0.1*meantm)
	{
	  flags2++;
	  sumfl=sumfl+fl1[i]-mean1-3.*rms1;
	}
    }
  for(i=1;i<=s3;i++)
    {
      if(fl1[i]>mean1+3.*rms1 && tme1[i]>=0.1*meantm)
	{
	  unf=unf+pow(fl1[i]-mean1-3.*rms1,2.);
	}
    }
  //printf("%f %d %f\n",unf,flags2,sumfl);
  unf=unf*flags2/(sumfl*sumfl);

  /////////////////////////////error of u//////////////////////////
  float *unf1,sumunf,sumunf1,erunf;
  unf1=vector(1,s3); sumunf1=0.; erunf=0.;
  
  for(j=1;j<=10;j++)
    {
      flags2=0; sumfl=0.; sumunf=0.;
      for(i=j;i<=s3;i=i+10)
	{
	  if(fl1[i]>mean1+3.*rms1 && tme1[i]>=0.1*meantm)
	    {
	      flags2++;
	      sumfl=sumfl+fl1[i]-mean1-3.*rms1;
	    }
	}
      for(i=j;i<=s3;i=i+10)
	{
	  if(fl1[i]>mean1+3.*rms1 && tme1[i]>=0.1*meantm)
	    {
	      sumunf=sumunf+pow(fl1[i]-mean1-3.*rms1,2.);
	    }
	}
      unf1[j]=sumunf*flags2/(sumfl*sumfl);
      //printf("%f %f\n",unf,unf1[j]);
      sumunf1=sumunf1+unf1[j];
    }
  sumunf1=sumunf1/10.;
  for(i=1;i<=10;i++)
    {
      erunf=erunf+pow(unf1[i]-sumunf1,2.);
    }
  erunf=sqrt(erunf/10.);
  if(erunf<0.01)
    {erunf=0.01;}
  //printf("%f %f\n",unf,erunf);

  *un=unf; *erun=erunf;
}

//////////////////////////////////////////////////////////////
/////////////////////////redshift scale///////////////////////
//////////////////////////////////////////////////////////////

float integral(float z)
{
  float x,intestep,intemin,integral1,suminte,dist,om,ol,h,pai;
  int i;
  h=0.7;
  om=0.3;
  ol=0.7;
  i=0;
  intemin=0.;
  integral1=0.;
  suminte=0.;
  intestep=z/1000;
  for (i=1;i<=1000;i++)
    {
      integral1=1/sqrt ((om*pow (1+i*intestep,3))+ol);
      suminte=suminte+integral1*intestep;
    }
  dist=suminte*3000./(h*(1+z));
  return dist;
}

//////////////////////////////////////////////////////////////
//////////////////////plot cluster image//////////////////////
//////////////////////////////////////////////////////////////
void plotobs(int id)
{
  FILE *fp;
  int i,RAH,RAM,decd,decm,nx,ny,dx,dy,flag;
  float RA,Dec,DecMin,DecMax,deltaDecMark,RAmin,RAmax,deltaRAmark,RAmark[4],RAS,DecMark[4],RAmin1,RAmax1,deltaRAmark1,RAmark1[4],centx,centy,modx,mody,ras,decs,rdt,c,w,pp,dis,asy,u,minx,maxx,miny,maxy,tagmin,maxfl,minrsd,maxrsd,mean,rms,tagmod,*px,*px1,*py,scale,ellip,tagsca; px=vector(1,4); px1=vector(1,4); py=vector(1,4);
  char filename[50],filename1[50],filename2[50],filename3[50],filename4[50],filename5[50],name[50],sign[5],**ticx,**ticy; ticx=mcvector(11,30); ticy=mcvector(11,30);
  sprintf(filename,"%d_plot.dat",id);
  sprintf(filename1,"%d_image.eps",id);
  sprintf(filename2,"%d_obj.dat",id);
  sprintf(filename3,"%d_objcntr.dat",id);
  sprintf(filename5,"%d_modcntrout.dat",id);
  fp=fopen(filename,"r");
  fscanf(fp,"%s %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f\n",name,&RA,&Dec,&centx,&centy,&ellip,&modx,&mody,&rdt,&scale,&tagsca,&c,&w,&pp,&dis,&asy,&u,&minx,&maxx,&miny,&maxy,&tagmin,&maxfl,&minrsd,&maxrsd,&mean,&rms,&tagmod);
  fclose(fp);
  

  DecMin = 3600.*Dec -(centy-miny)*1.476;
  DecMax = 3600.*Dec +(maxy-centy)*1.476;
  deltaDecMark = floor((DecMax-DecMin)/60. *0.5); // in arcmin

  for(i=1;i<=4;i++)
    {
      DecMark[i] = floor(DecMin/60)+1 + (i-1)*deltaDecMark ;   // in arcmin
      py[i] = (DecMark[i]*60 -  DecMin)/1.476 + miny; 
      
      if(DecMark[i] >=0 ) {strcpy(sign,"+"); }
      else {strcpy(sign,"-"); DecMark[i]=abs(DecMark[i]);}
      
      decd = floor( DecMark[i]/60 ) ;
      decm = floor( DecMark[i] -(60*decd) );
      decs = 0;
      sprintf(ticy[i],"%1s%02d:%02d:%02.0f",sign,decd,decm,decs);
    }
  

  RAmin = 3600. * RA - (maxx-centx)*1.476/cos((3.14159/180./3600.)*DecMin); //in arcsec
  RAmax = 3600. * RA + (centx-minx)*1.476/cos((3.14159/180./3600.)*DecMin); //in arcsec
  deltaRAmark = floor( (RAmax-RAmin) / 2. *1.2 / 75 ) *75;   // in arcsec
 
  for(i=1;i<=4;i++)  // for two marks in RA ****, find RAmark[1] and RAmark
    {
      RAmark[i] = floor(RAmin/15) + 1 + (i-1) * deltaRAmark /15 ; // in hour-sec
      RAmark[i]= (floor(RAmark[i]/5) +1 ) * 5 ;
      px[i] = maxx - (RAmark[i]*15 - RAmin) / (1.476/cos(DecMin*3.14159/180./3600.));

      RAH = floor( RAmark[i]/3600 ) ;  // in hour-H
      RAM = floor( RAmark[i]/60 - (60*RAH) ); // in hour-M
      RAS = RAmark[i] - RAH * 3600 - RAM*60;  // in hour-S
      sprintf(ticx[i],"%02d:%02d:%02.0f",RAH,RAM,RAS);
    }
 
  RAmin1 = 3600. * RA - (maxx-centx)*1.476/ cos( DecMax * 3.14159/180./3600.); //in arcsec
  RAmax1 = 3600. * RA + (centx-minx)*1.476/ cos( DecMax * 3.14159/180./3600.); //in arcsec
  deltaRAmark1 = floor( (RAmax1-RAmin1) / 2. *1.2 / 75 ) *75;   // in arcsec
  
  for(i=1;i<=4;i++)  
    {
      RAmark1[i] = floor(RAmin1/15) + 1 + (i-1) * deltaRAmark1 /15 ; // in hour-sec
      RAmark1[i]= (floor(RAmark1[i]/5) +1 ) * 5 ;
      px1[i] = maxx - (RAmark1[i]*15 - RAmin1) / (1.476/cos(DecMax*3.14159/180./3600.));
    }

  FILE * command=popen("gnuplot","w");  
  fprintf(command, "set terminal post enhanced color size 6,5 solid \"Times-Roman\" 16\n");
  fprintf(command, "set output \"%s\"\n",filename1);
  if(rdt>5)
    {fprintf(command, "set title \"%s (no z): Chandra image\" offset 0,-0.8 font \"Helvetica,14\" \n",name);}
  else
    {fprintf(command, "set title \"%s (z=%.4f): Chandra image\" offset 0,-0.8 font \"Helvetica,14\"\n",name,rdt);}
  fprintf(command, "set label 1 \"Contours: <S_{bg}>+3{/Symbol \\264}2^n{/Symbol \\163}, n = 0, 1, 2, 3, ...\" at graph 0.5,1.06 center front font \"Helvetica,14\"\n");
  fprintf(command, "set label 2 \"\" at %f,%f center point pt 1 ps 1.2 lw 1 lc rgbcolor \"white\" front\n",modx,mody);
  if(rdt<2) {
  if(rdt>0.01) {
  fprintf(command, "set label 3 \"500 kpc\" at graph 0.97,0.07 right front font \"Helvetica,14\" tc rgbcolor \"red\" \n");
  fprintf(command, "set arrow 1 nohead from %f,%f to %f,%f lw 0.5 lc rgbcolor \"red\" front\n",maxx-0.03*(maxx-minx),miny+0.03*(maxy-miny),maxx-0.03*(maxx-minx)-scale,miny+0.03*(maxy-miny));
  fprintf(command, "set arrow 2 nohead from %f,%f to %f,%f lw 0.5 lc rgbcolor \"red\" front\n",maxx-0.03*(maxx-minx),miny+0.01*(maxy-miny),maxx-0.03*(maxx-minx),miny+0.05*(maxy-miny));
  fprintf(command, "set arrow 3 nohead from %f,%f to %f,%f lw 0.5 lc rgbcolor \"red\" front\n",maxx-0.03*(maxx-minx)-scale,miny+0.01*(maxy-miny),maxx-0.03*(maxx-minx)-scale,miny+0.05*(maxy-miny));}
  else {
  fprintf(command, "set label 3 \"100 kpc\" at graph 0.97,0.07 right front font \"Helvetica,14\" tc rgbcolor \"red\" \n");
  fprintf(command, "set arrow 1 nohead from %f,%f to %f,%f lw 0.5 lc rgbcolor \"red\" front\n",maxx-0.03*(maxx-minx),miny+0.03*(maxy-miny),maxx-0.03*(maxx-minx)-scale/5.,miny+0.03*(maxy-miny));
  fprintf(command, "set arrow 2 nohead from %f,%f to %f,%f lw 0.5 lc rgbcolor \"red\" front\n",maxx-0.03*(maxx-minx),miny+0.01*(maxy-miny),maxx-0.03*(maxx-minx),miny+0.05*(maxy-miny));
  fprintf(command, "set arrow 3 nohead from %f,%f to %f,%f lw 0.5 lc rgbcolor \"red\" front\n",maxx-0.03*(maxx-minx)-scale/5.,miny+0.01*(maxy-miny),maxx-0.03*(maxx-minx)-scale/5.,miny+0.05*(maxy-miny));}    
  }
  fprintf(command, "set object 1 circle at %f,%f size %f front lw 0.5 fc rgbcolor \"red\" fs solid\n",minx+(maxx-minx)/40.+tagsca/2.,miny+(maxy-miny)/40.+tagsca/2.,tagsca/2.);
  fprintf(command, "set contour base\n");
  fprintf(command, "unset surface\n");
  fprintf(command, "set size square\n");
  fprintf(command, "set xlabel \"RA (J2000)\" offset 0,1 font \"Helvetica,14\"\n");
  fprintf(command, "set ylabel \"DEC (J2000)\" offset 1.5,0 font \"Helvetica,14\"\n");
  fprintf(command, "set xrange [%f:%f]\n",minx,maxx);
  fprintf(command, "set yrange [%f:%f]\n",miny,maxy);
  fprintf(command, "set cbrange [%f:%f]\n",tagmin,maxfl);
  fprintf(command, "set xtics (\"%s\" %f, \"%s\" %f, \"%s\" %f, \"%s\" %f) out nomirror offset 0,0.5 font \"Helvetica,14\"\n",ticx[1],px[1],ticx[2],px[2],ticx[3],px[3],ticx[4],px[4]);
  fprintf(command, "set x2tics (\"\" %f, \"\" %f, \"\" %f) out\n",px1[1],px1[2],px1[3]);
  fprintf(command, "set ytics (\"%s\" %f, \"%s\" %f, \"%s\" %f, \"%s\" %f) out mirror rotate by 90 center offset -0.5,0 font \"Helvetica,14\"\n",ticy[1],py[1],ticy[2],py[2],ticy[3],py[3],ticy[4],py[4]);
  fprintf(command, "set cblabel \"Brightness [{/Symbol \264}10^{-6}cnt s^{-1} arcsec^{-2}]\"offset 0.2,0 font \"Helvetica,14\"\n");
  fprintf(command, "set cbtics (\"10^{-4}\" 0.0001,\"10^{-3}\" 0.001,\"10^{-2}\" 0.01,\"10^{-1}\" 0.1, \"1\" 1, \"10\" 10, \"10^{2}\" 100, \"10^3\" 1000) offset -0.7,0 rotate by 90  font \"Helvetica,14\"\n");
  fprintf(command, "set logscale cb\n");
  fprintf(command, "set cntrparam levels discrete %f+%f*3.,%f+%f*6.,%f+%f*12.,%f+%f*24.,%f+%f*48.,%f+%f*96.,%f+%f*192.,%f+%f*384.,%f+%f*768.,%f+%f*1536.,%f+%f*3072.,%f+%f*6144.,%f+%f*12288.\n",mean,rms,mean,rms,mean,rms,mean,rms,mean,rms,mean,rms,mean,rms,mean,rms,mean,rms,mean,rms,mean,rms,mean,rms,mean,rms);
  fprintf(command, "set table '%s'\n",filename3);
  fprintf(command, "splot '%s'\n",filename2);
  fprintf(command, "unset table\n");
  fprintf(command, "set palette rgbformulae 22,13,-31\n");
  fprintf(command, "plot '%s' w image notitle, '%s' w lines lw 3 lc rgbcolor \"white\" lt 1 notitle, '%s' w lines lw 1 lc rgbcolor \"black\" notitle, '%s' w lines lw 3 lt 1 lc rgbcolor \"white\" notitle \n",filename2,filename3,filename3,filename5);
  fprintf(command, "set output\n");
  fflush(command);
  fclose(command);
}

void plotmod(int id)
{
  FILE *fp;
  int i,RAH,RAM,decd,decm,nx,ny,dx,dy,flag;
  float RA,Dec,DecMin,DecMax,deltaDecMark,RAmin,RAmax,deltaRAmark,RAmark[4],RAS,DecMark[4],RAmin1,RAmax1,deltaRAmark1,RAmark1[4],centx,centy,modx,mody,ras,decs,rdt,c,w,pp,dis,asy,u,minx,maxx,miny,maxy,tagmin,maxfl,minrsd,maxrsd,mean,rms,tagmod,*px,*px1,*py,scale,ellip,tagsca; px=vector(1,4); px1=vector(1,4); py=vector(1,4);
  char filename[50],filename1[50],filename2[50],filename3[50],filename4[50],name[50],sign[5],**ticx,**ticy; ticx=mcvector(11,30); ticy=mcvector(11,30);
  sprintf(filename,"%d_plot.dat",id);
  sprintf(filename1,"%d_model.eps",id);
  sprintf(filename2,"%d_mod.dat",id);
  sprintf(filename3,"%d_modcntr.dat",id);
  sprintf(filename4,"%d_modcntrout.dat",id);
  fp=fopen(filename,"r");
  fscanf(fp,"%s %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f\n",name,&RA,&Dec,&centx,&centy,&ellip,&modx,&mody,&rdt,&scale,&tagsca,&c,&w,&pp,&dis,&asy,&u,&minx,&maxx,&miny,&maxy,&tagmin,&maxfl,&minrsd,&maxrsd,&mean,&rms,&tagmod);
  fclose(fp);

  DecMin = 3600.*Dec-(centy-miny)*1.476;
  DecMax = 3600.*Dec+(maxy-centy)*1.476;
  deltaDecMark = floor((DecMax-DecMin)/60. *0.6); // in arcmin

  for(i=1;i<=4;i++)
    {
      DecMark[i] = floor(DecMin/60)+1 + (i-1)*deltaDecMark ;   // in arcmin
      py[i] = (DecMark[i]*60 -  DecMin)/1.476 + miny; 
      
      if(DecMark[i] >=0 ) {strcpy(sign,"+"); }
      else {strcpy(sign,"-"); DecMark[i]=abs(DecMark[i]);}
      
      decd = floor( DecMark[i]/60 ) ;
      decm = floor( DecMark[i] -(decd*60) );
      decs = 0;
      sprintf(ticy[i],"%1s%02d:%02d:%02.0f",sign,decd,decm,decs);
    }

  RAmin = 3600. * RA - (maxx-centx)*1.476/cos(DecMin*3.14159/180./3600.); //in arcsec
  RAmax = 3600. * RA + (centx-minx)*1.476/cos(DecMin*3.14159/180./3600.); //in arcsec
  deltaRAmark = floor( (RAmax-RAmin) / 2. *1.2 / 75 ) *75;   // in arcsec
  
  for(i=1;i<=4;i++)  // for two marks in RA ****, find RAmark[1] and RAmark
    {
      RAmark[i] = floor(RAmin/15) + 1 + (i-1) * deltaRAmark /15 ; // in hour-sec
      RAmark[i]= (floor(RAmark[i]/5) +1 ) * 5 ;
      px[i] = maxx - (RAmark[i]*15 - RAmin) / (1.476/cos(DecMin*3.14159/180./3600.));

      RAH = floor( RAmark[i]/3600 ) ;  // in hour-H
      RAM = floor( RAmark[i]/60 -(RAH*60) ); // in hour-M
      RAS = RAmark[i] - RAH * 3600 - RAM*60;  // in hour-S
      sprintf(ticx[i],"%02d:%02d:%02.0f",RAH,RAM,RAS);
    }

  RAmin1 = 3600. * RA - (maxx-centx)*1.476/cos(DecMax*3.14159/180./3600.); //in arcsec
  RAmax1 = 3600. * RA + (centx-minx)*1.476/cos(DecMax*3.14159/180./3600.); //in arcsec
  deltaRAmark1 = floor( (RAmax1-RAmin1) / 2. *1.2 / 75 ) *75;   // in arcsec
  
  for(i=1;i<=4;i++)  
    {
      RAmark1[i] = floor(RAmin1/15) + 1 + (i-1) * deltaRAmark1 /15 ; // in hour-sec
      RAmark1[i]= (floor(RAmark1[i]/5) +1 ) * 5 ;
      px1[i] = maxx - (RAmark1[i]*15 - RAmin1) / (1.476/cos(DecMax*3.14159/180./3600.));
    }
  FILE * command=popen("gnuplot","w");  
  fprintf(command, "set terminal post enhanced color size 6,5 solid \"Times-Roman\" 16\n");
  fprintf(command, "set output \"%s\"\n",filename1);
  fprintf(command, "set title \"%s: Fitting model\" offset 0,-0.8 font \"Helvetica,14\" \n",name);
  fprintf(command, "set contour base\n");
  fprintf(command, "unset surface\n");
  fprintf(command, "set size square\n");
  fprintf(command, "set xlabel \"RA (J2000)\" offset 0,1 font \"Helvetica,14\"\n");
  fprintf(command, "set ylabel \"DEC (J2000)\" offset 1.5,0 font \"Helvetica,14\"\n");
  fprintf(command, "set label 1 \"Contours: <S_{bg}>+3{/Symbol \\264}2^n{/Symbol \\163}, n = 0, 1, 2, 3, ...\" at graph 0.5,1.06 center front font \"Helvetica,14\"\n");
  fprintf(command, "set label 2 \"\" at %f,%f center point pt 1 ps 1.2 lw 1 lc rgbcolor \"white\" front\n",modx,mody);

  fprintf(command, "set label 3 \" e = %.2f\" at graph 0.97,0.07 right front font \"Helvetica,14\" tc rgbcolor \"red\" \n",ellip);

  fprintf(command, "set xrange [%f:%f]\n",minx,maxx);
  fprintf(command, "set yrange [%f:%f]\n",miny,maxy);
  fprintf(command, "set cbrange [%f:%f]\n",tagmin,maxfl);
  fprintf(command, "set xtics (\"%s\" %f, \"%s\" %f, \"%s\" %f, \"%s\" %f) out nomirror offset 0,0.5 font \"Helvetica,14\"\n",ticx[1],px[1],ticx[2],px[2],ticx[3],px[3],ticx[4],px[4]);
  fprintf(command, "set x2tics (\"\" %f, \"\" %f, \"\" %f) out\n",px1[1],px1[2],px1[3]);
  fprintf(command, "set ytics (\"%s\" %f, \"%s\" %f, \"%s\" %f, \"%s\" %f) out mirror rotate by 90 center offset -0.5,0 font \"Helvetica,14\"\n",ticy[1],py[1],ticy[2],py[2],ticy[3],py[3],ticy[4],py[4]);
  fprintf(command, "set cblabel \"Brightness [{/Symbol \264}10^{-6}cnt s^{-1} arcsec^{-2}]\"offset 0.2,0  font \"Helvetica,14\"\n");
  fprintf(command, "set cbtics (\"10^{-4}\" 0.0001,\"10^{-3}\" 0.001,\"10^{-2}\" 0.01,\"10^{-1}\" 0.1, \"1\" 1, \"10\" 10, \"10^{2}\" 100, \"10^3\" 1000) offset -0.7,0 rotate by 90 font \"Helvetica,14\"\n");
  fprintf(command, "set logscale cb\n");
  fprintf(command, "set cntrparam levels discrete %f+%f*3.,%f+%f*6.,%f+%f*12.,%f+%f*24.,%f+%f*48.,%f+%f*96.,%f+%f*192.,%f+%f*384.,%f+%f*768.,%f+%f*1536.,%f+%f*3072.,%f+%f*6144.,%f+%f*12288.\n",mean,rms,mean,rms,mean,rms,mean,rms,mean,rms,mean,rms,mean,rms,mean,rms,mean,rms,mean,rms,mean,rms,mean,rms,mean,rms);
  fprintf(command, "set table '%s'\n",filename3);
  fprintf(command, "splot '%s'\n",filename2);
  fprintf(command, "unset table\n");
  fprintf(command, "set cntrparam levels discrete %f\n",tagmod);
  fprintf(command, "set table '%s'\n",filename4);
  fprintf(command, "splot '%s'\n",filename2);
  fprintf(command, "unset table\n");
  
  fprintf(command, "set palette rgbformulae 22,13,-31\n");
  fprintf(command, "plot '%s' w image notitle, '%s' w lines lw 3 lc rgbcolor \"white\" notitle, '%s' w lines lw 1 lc rgbcolor \"black\" notitle\n",filename2,filename3,filename3);
  fprintf(command, "set output\n");
  fflush(command);
  fclose(command);
}


void plotrsd(int id)
{
  FILE *fp;
  int i,RAH,RAM,decd,decm,nx,ny,dx,dy,flag;
  float RA,Dec,DecMin,DecMax,deltaDecMark,RAmin,RAmax,deltaRAmark,RAmark[4],RAS,DecMark[4],RAmin1,RAmax1,deltaRAmark1,RAmark1[4],centx,centy,modx,mody,ras,decs,rdt,c,w,pp,dis,asy,u,minx,maxx,miny,maxy,tagmin,maxfl,minrsd,maxrsd,mean,rms,tagmod,*px,*px1,*py,scale,ellip,tagsca; px=vector(1,4); px1=vector(1,4); py=vector(1,4);
  char filename[50],filename1[50],filename2[50],filename3[50],filename4[50],filename5[50],name[50],sign[5],**ticx,**ticy; ticx=mcvector(11,30); ticy=mcvector(11,30);
  sprintf(filename,"%d_plot.dat",id);
  sprintf(filename1,"%d_residual.eps",id);
  sprintf(filename2,"%d_rsd.dat",id);
  sprintf(filename3,"%d_rsdcntr1.dat",id);
  sprintf(filename4,"%d_rsdcntr2.dat",id);
  sprintf(filename5,"%d_modcntrout.dat",id);
  fp=fopen(filename,"r");
  fscanf(fp,"%s %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f\n",name,&RA,&Dec,&centx,&centy,&ellip,&modx,&mody,&rdt,&scale,&tagsca,&c,&w,&pp,&dis,&asy,&u,&minx,&maxx,&miny,&maxy,&tagmin,&maxfl,&minrsd,&maxrsd,&mean,&rms,&tagmod);
  fclose(fp);

  DecMin = 3600.*Dec-(centy-miny)*1.476;
  DecMax = 3600.*Dec+(maxy-centy)*1.476;
  deltaDecMark = floor((DecMax-DecMin)/60. *0.6); // in arcmin

  for(i=1;i<=4;i++)
    {
      DecMark[i] = floor(DecMin/60)+1 + (i-1)*deltaDecMark ;   // in arcmin
      py[i] = (DecMark[i]*60 -  DecMin)/1.476 + miny; 
      
      if(DecMark[i] >=0 ) {strcpy(sign,"+"); }
      else {strcpy(sign,"-"); DecMark[i]=abs(DecMark[i]);}
      
      decd = floor( DecMark[i]/60 ) ;
      decm = floor( DecMark[i] -(decd*60) );
      decs = 0;
      sprintf(ticy[i],"%1s%02d:%02d:%02.0f",sign,decd,decm,decs);
    }

  RAmin = 3600. * RA - (maxx-centx)*1.476/cos(DecMin*3.14159/180./3600.); //in arcsec
  RAmax = 3600. * RA + (centx-minx)*1.476/cos(DecMin*3.14159/180./3600.); //in arcsec
  deltaRAmark = floor( (RAmax-RAmin) / 2. *1.2 / 75 ) *75;   // in arcsec
  
  for(i=1;i<=4;i++)  // for two marks in RA ****, find RAmark[1] and RAmark
    {
      RAmark[i] = floor(RAmin/15) + 1 + (i-1) * deltaRAmark /15 ; // in hour-sec
      RAmark[i]= (floor(RAmark[i]/5) +1 ) * 5 ;
      px[i] = maxx - (RAmark[i]*15 - RAmin) / (1.476/cos(DecMin*3.14159/180./3600.));

      RAH = floor( RAmark[i]/3600 ) ;  // in hour-H
      RAM = floor( RAmark[i]/60 -(RAH*60) ); // in hour-M
      RAS = RAmark[i] - RAH * 3600 - RAM*60;  // in hour-S
      sprintf(ticx[i],"%02d:%02d:%02.0f",RAH,RAM,RAS);
    }

  RAmin1 = 3600. * RA - (maxx-centx)*1.476/cos(DecMax*3.14159/180./3600.); //in arcsec
  RAmax1 = 3600. * RA + (centx-minx)*1.476/cos(DecMax*3.14159/180./3600.); //in arcsec
  deltaRAmark1 = floor( (RAmax1-RAmin1) / 2. *1.2 / 75 ) *75;   // in arcsec
  
  for(i=1;i<=4;i++)  
    {
      RAmark1[i] = floor(RAmin1/15) + 1 + (i-1) * deltaRAmark1 /15 ; // in hour-sec
      RAmark1[i]= (floor(RAmark1[i]/5) +1 ) * 5 ;
      px1[i] = maxx - (RAmark1[i]*15 - RAmin1) / (1.476/cos(DecMax*3.14159/180./3600.));
    }
  FILE * command=popen("gnuplot","w");  
  fprintf(command, "set terminal post enhanced color size 6,5 \"Times-Roman\" 16\n");
  fprintf(command, "set output \"%s\"\n",filename1);
  fprintf(command, "set title \"%s: Residual map\" offset 0,-0.8 font \"Helvetica,14\"\n",name);
  fprintf(command, "set contour base\n");
  fprintf(command, "unset surface\n");
  fprintf(command, "set size square\n");
  fprintf(command, "set xlabel \"RA (J2000)\" offset 0,1 font \"Helvetica,14\"\n");
  fprintf(command, "set ylabel \"DEC (J2000)\" offset 1.5,0 font \"Helvetica,14\"\n");
  fprintf(command, "set label 1 \"Contours: {/Symbol \\261}3{/Symbol \\264}2^n{/Symbol \\163}, n = 0, 1, 2, 3, ...\" at graph 0.5,1.06 center front font \"Helvetica,14\"\n");
  fprintf(command, "set label 2 \"\" at %f,%f center point pt 1 ps 1.2 lw 1 lc rgbcolor \"white\" front\n",modx,mody);
  if(rdt<2){
  if(rdt>0.01) {
  fprintf(command, "set label 3 \"500 kpc\" at graph 0.97,0.07 right front font \"Helvetica,14\" tc rgbcolor \"red\" \n");
  fprintf(command, "set arrow 1 nohead from %f,%f to %f,%f lw 0.5 lc rgbcolor \"red\" front\n",maxx-0.03*(maxx-minx),miny+0.03*(maxy-miny),maxx-0.03*(maxx-minx)-scale,miny+0.03*(maxy-miny));
  fprintf(command, "set arrow 2 nohead from %f,%f to %f,%f lw 0.5 lc rgbcolor \"red\" front\n",maxx-0.03*(maxx-minx),miny+0.01*(maxy-miny),maxx-0.03*(maxx-minx),miny+0.05*(maxy-miny));
  fprintf(command, "set arrow 3 nohead from %f,%f to %f,%f lw 0.5 lc rgbcolor \"red\" front\n",maxx-0.03*(maxx-minx)-scale,miny+0.01*(maxy-miny),maxx-0.03*(maxx-minx)-scale,miny+0.05*(maxy-miny));}
  else {
  fprintf(command, "set label 3 \"100 kpc\" at graph 0.97,0.07 right front font \"Helvetica,14\" tc rgbcolor \"red\" \n");
  fprintf(command, "set arrow 1 nohead from %f,%f to %f,%f lw 0.5 lc rgbcolor \"red\" front\n",maxx-0.03*(maxx-minx),miny+0.03*(maxy-miny),maxx-0.03*(maxx-minx)-scale/5.,miny+0.03*(maxy-miny));
  fprintf(command, "set arrow 2 nohead from %f,%f to %f,%f lw 0.5 lc rgbcolor \"red\" front\n",maxx-0.03*(maxx-minx),miny+0.01*(maxy-miny),maxx-0.03*(maxx-minx),miny+0.05*(maxy-miny));
  fprintf(command, "set arrow 3 nohead from %f,%f to %f,%f lw 0.5 lc rgbcolor \"red\" front\n",maxx-0.03*(maxx-minx)-scale/5.,miny+0.01*(maxy-miny),maxx-0.03*(maxx-minx)-scale/5.,miny+0.05*(maxy-miny));}    
  }
  fprintf(command, "set object 1 circle at %f,%f size %f front lw 0.5 fc rgbcolor \"red\" fs solid\n",minx+(maxx-minx)/40.+tagsca/2.,miny+(maxy-miny)/40.+tagsca/2.,tagsca/2.);
  fprintf(command, "set cblabel \"Residual: {/Symbol \\264}{/Symbol \\163}\"offset 0,0 font \"Helvetica,14\"\n");
  fprintf(command, "set xrange [%f:%f]\n",minx,maxx);
  fprintf(command, "set yrange [%f:%f]\n",miny,maxy);
  fprintf(command, "set cbrange [%f:%f]\n",minrsd,maxrsd);
  fprintf(command, "set xtics (\"%s\" %f, \"%s\" %f, \"%s\" %f, \"%s\" %f) out nomirror offset 0,0.5 font \"Helvetica,14\"\n",ticx[1],px[1],ticx[2],px[2],ticx[3],px[3],ticx[4],px[4]);
  fprintf(command, "set x2tics (\"\" %f, \"\" %f, \"\" %f) out\n",px1[1],px1[2],px1[3]);
  fprintf(command, "set ytics (\"%s\" %f, \"%s\" %f, \"%s\" %f, \"%s\" %f) out mirror rotate by 90 center offset -0.5,0 font \"Helvetica,14\"\n",ticy[1],py[1],ticy[2],py[2],ticy[3],py[3],ticy[4],py[4]);
  fprintf(command, "unset logscale cb\n");
  if(maxrsd+fabs(minrsd)<24*rms)
    {fprintf(command, "set cbtics (\"24\" 24*%f, \"12\" 12*%f, \"6\" 6*%f, \"3\" 3*%f, \"0\" 0, \"-3\" -3*%f, \"-6\" -6*%f, \"-12\" -12*%f, \"-24\" -24*%f) offset -0.5,0\n",rms,rms,rms,rms,rms,rms,rms,rms);}
  else if (maxrsd+fabs(minrsd)<48*rms)
    {fprintf(command, "set cbtics (\"48\" 48*%f, \"24\" 24*%f, \"12\" 12*%f, \"6\" 6*%f, \"0\" 0, \"-6\" -6*%f, \"-12\" -12*%f, \"-24\" -24*%f, \"-48\" -48*%f) offset -0.5,0 font \"Helvetica,14\"\n",rms,rms,rms,rms,rms,rms,rms,rms);}
  else if (maxrsd+fabs(minrsd)<96*rms)
    {fprintf(command, "set cbtics (\"96\" 96*%f, \"48\" 48*%f, \"24\" 24*%f, \"12\" 12*%f, \"0\" 0, \"-12\" -12*%f, \"-24\" -24*%f, \"-48\" -48*%f, \"-96\" -96*%f) offset -0.5,0 font \"Helvetica,14\"\n",rms,rms,rms,rms,rms,rms,rms,rms);}
  else
    {fprintf(command, "set cbtics (\"192\" 192*%f, \"96\" 96*%f, \"48\" 48*%f, \"0\" 0, \"-48\" -48*%f, \"-96\" -96*%f, \"-192\" -192*%f) offset -0.5,0 font \"Helvetica,14\"\n",rms,rms,rms,rms,rms,rms,rms,rms);}
  fprintf(command, "set cntrparam levels discrete 3*%f,6*%f,12*%f,24*%f,48*%f\n",rms,rms,rms,rms,rms);
  fprintf(command, "set table '%s'\n",filename3);
  fprintf(command, "splot '%s'\n",filename2);
  fprintf(command, "set cntrparam levels discrete -3*%f,-6*%f,-12*%f,-24*%f,-48*%f\n",rms,rms,rms,rms,rms);
  fprintf(command, "set table '%s'\n",filename4);
  fprintf(command, "splot '%s'\n",filename2);
  fprintf(command, "unset table\n");
  fprintf(command, "set palette rgbformulae 22,13,-31\n");
  fprintf(command, "plot '%s' w image notitle, '%s' w lines lw 3 lc rgbcolor \"white\" notitle, '%s' w lines lw 1 lt 1 lc rgbcolor \"black\" notitle, '%s' w lines lw 1 lc rgbcolor \"white\" notitle, '%s' w lines lt 0 lw 1 lc rgbcolor \"black\" notitle, '%s' w lines lw 3 lt 0 lc rgbcolor \"white\" notitle \n",filename2,filename3,filename3,filename4,filename4,filename5);
  fprintf(command, "set output\n");
  fflush(command);
  fclose(command);
}
