#include "stdio.h"
#include "time.h"
#include "stdlib.h"
#include "string.h"
#include "math.h"
#include "fitsio.h"  //**for reading fits file, from fitsio package  
#include "nrutil.h"     //**utility file from the Numerical Recipes
#include "nrutil.c"     //**utility file from the Numerical Recipes
#include "nr.h"         //**utility file from the Numerical Recipes
#include "funcs2D.c"   //**modified code from the Numerical Recipes 
#include "mrqmin2D1.c" //**modified code from the Numerical Recipes 
#include "mrqcof2D1.c" //**modified code from the Numerical Recipes 
#include "mrqmin2D2.c" //**modified code from the Numerical Recipes 
#include "mrqcof2D2.c" //**modified code from the Numerical Recipes 
#include "covsrt2D.c"  //**modified code from the Numerical Recipes 
#include "gammln.c"    //**modified code from the Numerical Recipes 
#include "gaussj.c"    //**modified code from the Numerical Recipes 

void main(int argc, char *argv[]) //the main program
{

  ///////////////////////////////////////////////////////////////////
  ////                  print           tips                    /////
  ///////////////////////////////////////////////////////////////////
  while(argc<3)
    {
      printf("\n ############################################################\n");
      printf(" Please prepare a file named \"clstr.inf\" with format:\n");
      printf(" **cluster name, RA & Dec(J2000), redshift, XMM-Newton ObsID,\n");
      printf(" **initial center of clusters in X & Y-axis (in pixel),\n");
      printf(" **center of background region in X & Y-axis (in pixel), e.g.:\n");
      printf(" A7   2.93500  32.41694  0.1062  0827021201  427  453  670  372\n");
      printf(" A13  3.40958 -19.50222  0.0943  0200270101  362  430  700  447\n");
      printf("\n and input the cluster image with option:\n");
      printf(" -f clstr_name.fits\n");
      printf("\n Then you can get dynamical parameters with:\n");
      printf(" -c: the concentration index,\n");
      printf(" -w: the centroid shift,\n");
      printf(" -p: the power ratio,\n");
      printf(" -k: the profile parameter,\n");
      printf(" -a: the asymmetry factor,\n");
      printf(" -d: the morphology index,\n");
      printf("\n and obtain images with -plot for:\n");
      printf(" observation image: clstr_obs.eps;\n");
      printf(" and fitting model: clstr_mod.eps.\n");
      printf("\n An example:\n");
      printf(" ./getDynamic -f clstr_name.fits -c -w -p -d -plot\n");
      printf(" ############################################################\n\n");
      exit(0);
    }

  ///////////////////////////////////////////////////////////////////
  ////             judge   the   input   commands               /////
  ///////////////////////////////////////////////////////////////////
  int i=1,flagc=0,flagw=0,flagp=0,flagk=0,flaga=0,flagd=0,flagplot=0,flagf=0;
  char objname[100];
  while(i<argc)
    {
      if(strcmp(argv[i],"-f")==0)  //judging whether the cluster image is inputted;
 	{
	  flagf=1;
	  strcpy(objname,argv[i+1]);
	}
      if(strcmp(argv[i],"-c")==0)  //judging whether calculate the concentration index;
	{
	  flagc=1;
	}
      if(strcmp(argv[i],"-w")==0)  //judging whether calculate the central shift;
	{
	  flagw=1;
	}
      if(strcmp(argv[i],"-p")==0)  //judging whether calculate the power ratio;
 	{
	  flagp=1;
	}
      if(strcmp(argv[i],"-k")==0)  //judging whether calculate the profile parameter;
	{
	  flagk=1;
	}
      if(strcmp(argv[i],"-a")==0)  //judging whether calculate the asymmetry factor;
	{
	  flaga=1;
	}
      if(strcmp(argv[i],"-d")==0)  //judging whether calculate the morphology index;
 	{
	  flagd=1;
	}
      if(strcmp(argv[i],"-plot")==0)  //judging whether plot cluster images;
 	{
	  flagplot=1;
	}
      i++;
    }

  if(flagf==0)
    {
      printf("\n Please input the cluster image with: -f clstr_name.fits!\n");
      exit(0);
    }

  int k;
  k=strlen(objname);
  for(i=0;i<=5;i++)     //remove the file suffix, e.g., A7.fits ==> A7
    {
      objname[k-i]='\0';
    }

  /////////////////////////////////////////////////////////////////////
  ////       read     parameters    from    clstr.inf             /////
  /////////////////////////////////////////////////////////////////////
  FILE *fp;
  int flag=0;
  float ra,dec,z,cntx0,cnty0,bgx0,bgy0,scale30,scale500,psize;
  char name[50],obsid[20];
  float integral(float z);
  psize=2.5;   // pixel size in arcsec

  fp=fopen("clstr.inf","r");
  for(i=1;i<=2000;i++)
    {
      fscanf(fp,"%s %f %f %f %s %f %f %f %f",name,&ra,&dec,&z,obsid,&cntx0,&cnty0,&bgx0,&bgy0);
      if(strcmp(objname,name)==0) //search the input cluster in the clstr.inf
	{
	  printf("\n Information of the galaxy cluster:\n");
	  printf(" #######################################################\n");
	  printf(" ## Cluster Name: %-16s  ObsID: %s ##\n",name,obsid);
	  printf(" ##   RA: %9.5f    Dec: %9.5f     z: %6.4f   ##\n",ra,dec,z);
	  printf(" #######################################################\n\n");
	  if(z<5) //for clusters with measured redshifts
	    {
	      scale30=30./((integral(z)*1000.*(3.14159/(180.*3600.)))*psize);   //N_pixel = 30 kpc
	      scale500=500./((integral(z)*1000.*(3.14159/(180.*3600.)))*psize); //N_pixel = 500 kpc
	    }
	  else // for clusters with no available redshift
	    {
	      scale30=2.; //smooth to 5 arcsec for clusters with no-z
	      scale500=2.*(500./30.);
	    }
	  flag=1;
	  break;
	}
    }
  fclose(fp);

  //printf("%.2f %.2f\n",scale30,scale500);

  if(flag ==0)
    {
      printf("\nCan not find parameters in the input file for %s!\nExit.\n",objname);
      exit(0);
    }
  
  /////////////////////////////////////////////////////////////////////
  /////            read    cluster   fits    image                 ////
  /////////////////////////////////////////////////////////////////////
  int N_tot,nrows[1],ncols[1];
  float cntra[1],cntdec[1],*posx,*posy,*flux,*flx,*x,*y,*fl;
  char file[100];
  void readfits(char fitsname[], float psize, float cntx0, float cnty0, float *cntra, float *cntdec, int *nrow, int *ncol, float posx[], float posy[], float flux[], float flx[]);
  //read the fits
  N_tot=2048*2048;
  posx=vector(1,N_tot); posy=vector(1,N_tot); 
  flux=vector(1,N_tot); flx=vector(1,N_tot);
  strcpy(file, objname);
  strcat(file, ".fits");
  readfits(file,psize,cntx0,cnty0,cntra,cntdec,nrows,ncols,posx,posy,flux,flx);

  //save in arrays with smaller memory consumption
  N_tot=nrows[0]*ncols[0];
  x=vector(1,N_tot); y=vector(1,N_tot); fl=vector(1,N_tot);

  for(i=1;i<=N_tot;i++)
    {
      x[i]=posx[i];
      y[i]=posy[i];
      fl[i]=flux[i];
    }

  /////////////////////////////////////////////////////////////////////
  ////    calculate the mean flux & rms of backgrond region        ////
  /////////////////////////////////////////////////////////////////////
  void backgrnd(int ndata, float x[], float y[], float fl[], float bgx0, float bgy0, float *mean, float *rms);
  float mean[1],rms[1];

  backgrnd(N_tot,x,y,fl,bgx0,bgy0,mean,rms);

  /////////////////////////////////////////////////////////////////////
  ////        select the region according to the cluster size      ////
  ////                and save data in the region                  ////
  /////////////////////////////////////////////////////////////////////
  void maprgn(int ndata, float x[], float y[], float fl[], float cntx0, float cnty0, float scale500, float z, float *mean, float *rms, float *r_map, float *minx, float *miny, float *maxx, float *maxy, float *minfl, float *maxfl, float *peak_x, float *peak_y);
  float r_map[1],minx[1],miny[1],maxx[1],maxy[1],minfl[1],maxfl[1],peak_x[1],peak_y[1];
  maprgn(N_tot,x,y,fl,cntx0,cnty0,scale500,z,mean,rms,r_map,minx,miny,maxx,maxy,minfl,maxfl,peak_x,peak_y);
  //printf("%f %f %f\n",*peak_x,*peak_y,*maxfl);

  //save data in new arrays within the selected region;
  int N_map=0;
  float *x_map,*y_map,*fl_map;
  x_map=vector(1,N_tot); y_map=vector(1,N_tot); fl_map=vector(1,N_tot);

  for(i=1;i<=N_tot;i++)
    {
      if(x[i]>=*minx && x[i]<=*maxx && y[i]>=*miny && y[i]<=*maxy)
	{
	  N_map++;
	  x_map[N_map]=x[i];
	  y_map[N_map]=y[i];
	  fl_map[N_map]=fl[i];
	}
    }

  /////////////////////////////////////////////////////////////////////
  ////       fitting   the   image   with   2D-beta    model      /////  
  /////////////////////////////////////////////////////////////////////
  void imagefit(char name[], int N_map, float x_map[], float y_map[], float fl_map[], float *maxfl, float *minfl, float *r_map, float cntx0, float cnty0, float *mean, float *rms, float coeff[], float *chi, float fl_mod[], float *fl_border, float *fit_peak_x0, float *fit_peak_y0, float *kappa);
  float *coeff,chi[1],*fl_mod,fl_border[1],fit_peak_x0[1],fit_peak_y0[1],kappa[1];
  coeff=vector(1,8); fl_mod=vector(1,N_map);
 
  imagefit(objname,N_map,x_map,y_map,fl_map,maxfl,minfl,r_map,cntx0,cnty0,mean,rms,coeff,chi,fl_mod,fl_border,fit_peak_x0,fit_peak_y0,kappa);

  printf("A=%.2f r0=%.2f beta=%.2f ellip=%.2f theta=%.2f x0=%.2f y0=%.2f bg=%.2f kappa=%.2f\n",coeff[1],coeff[2],coeff[3],coeff[4],coeff[5],coeff[6],coeff[7],coeff[8],*kappa);
  //get the coordinate of the fitted center
  
  /////////////////////////////////////////////////////////////////////
  ////       calculate     dynamical    parameters                /////
  /////////////////////////////////////////////////////////////////////

  //calculate: c (the concentration index)
  void calculate_c(float z, int ndata, float x[], float y[], float fl[], float *mean, float *rms, float fit_x0, float fit_y0, float scale500, float *ci, float *erci);
  float ci[1],erci[1];
  if(flagc==1)
    {
      calculate_c(z,N_map,x_map,y_map,fl_map,mean,rms,coeff[6],coeff[7],scale500,ci,erci);
      //printf("%.2f %.2f\n",*ci,*erci);
    }
  else
    {
      *ci=0.; *erci=0.;
    }

  //calculate: w (the centroid shift)
  void calculate_w(float z, int ndata, float x[], float y[], float fl[], float *mean, float fit_x0, float fit_y0, float scale500, float *peak_x, float *peak_y, float *maxfl, float *w, float *erw);
  float w[1],erw[1];
  if(flagw==1 && *ci !=0)
    {
      calculate_w(z,N_tot,x,y,fl,mean,coeff[6],coeff[7],scale500,peak_x,peak_y,maxfl,w,erw);
      //printf("%.2f %.2f\n",*w,*erw);
    }
  else if(flagw==0)
    {
      *w=0.; *erw=0.;
    }
  else
    {*w=0.; *erw=0.;}

  //calculate: p (the power ratio)
  void calculate_p(float z, int ndata, float x[], float y[], float fl[], float *mean, float fit_x0, float fit_y0, float scale500, float *pp, float *erpp);
  float pp[1],erpp[1];
  if(flagp==1 && *ci !=0)
    {
      calculate_p(z,N_map,x_map,y_map,fl_map,mean,coeff[6],coeff[7],scale500,pp,erpp);
      //printf("%.2f %.2f\n",*pp,*erpp);
    }
  else if(flagp==0)
    {
      *pp=0.; *erpp=0.;
    }
  else
    {*pp=0.; *erpp=0.;}
  
  //calculate: a (the asymmetry factor)
  void calculate_a(int ndata, float x[], float y[], float fl[], float flx[], float fl_mod[], int nrows, float *fl_border, float fit_x0, float fit_y0, float *af, float *eraf);
  float af[1],eraf[1];
  if(flaga==1)
    {
      calculate_a(N_map,x_map,y_map,fl_map,flx,fl_mod,nrows[0],fl_border,coeff[6],coeff[7],af,eraf);
      //printf("%.2f %.2f\n",*af,*eraf);
    }
  else
    {
      *af=0.; *eraf=0.;
    }

  //calculate: d (the asymmetry factor)
  float dlt[1],erdlt[1];
  if(flagd==1)
    {
      dlt[0]=0.68*af[0]+0.73*kappa[0]+0.21;
      erdlt[0]=0.68*eraf[0];
      if(erdlt[0]<0.01)
	{erdlt[0]=0.01;}
      //printf("%.2f %.2f\n",*dlt,*erdlt);
    }
  else
    {
      *dlt=0.; *erdlt=0.;
    }

  printf("c=%.2f %.2f w=%.2f %.2f p=%.2f %.2f k=%.2f a=%.2f %.2f d=%.2f %.2f\n",*ci,*erci,*w,*erw,*pp,*erpp,*kappa,*af,*eraf,*dlt,*erdlt);

  char filename[50];
  sprintf(filename,"%s_parameter.dat",name);
  fp=fopen(filename,"w+");
  fprintf(fp,"%-20s %9.5f %9.5f %6.4f %5.2f %5.2f %5.2f %5.2f %5.2f %5.2f %5.2f %5.2f %5.2f %5.2f %5.2f\n",name,ra,dec,z,*ci,*erci,*w,*erw,*pp,*erpp,*kappa,*af,*eraf,*dlt,*erdlt);
  fclose(fp);

  /////////////////////////////////////////////////////////////////////
  ////            plot         cluster            images           ////
  /////////////////////////////////////////////////////////////////////
  
  //plot fitting-model image
  void plotmod(char name[], float psize, float *cntra, float *cntdec, float cntx0, float cnty0, float ellip, float fit_x0, float fit_y0, float *minx, float *maxx, float *miny, float *maxy, float *minfl, float *maxfl, float *mean, float *rms, float *fl_border);
  if(flagplot==1)
    {
      printf("\n......plotting images......\n");
      plotmod(objname,psize,cntra,cntdec,cntx0,cnty0,coeff[4],coeff[6],coeff[7],minx,maxx,miny,maxy,minfl,maxfl,mean,rms,fl_border);
    }
  
  //plot observation image
void plotobs(char name[], float z, float psize, float *cntra, float *cntdec, float cntx0, float cnty0, float fit_x0, float fit_y0, float *minx, float *maxx, float *miny, float *maxy, float *minfl, float *maxfl, float *mean, float *rms, float scale500, float scale30, float *ci, float *w, float *pp, float *dlt);
  if(flagplot==1)
    {
      plotobs(objname,z,psize,cntra,cntdec,cntx0,cnty0,coeff[6],coeff[7],minx,maxx,miny,maxy,minfl,maxfl,mean,rms,scale500,scale30,ci,w,pp,dlt);
   }
}


//////////////////////////////////////////////////////////////
/////////////      functional  subroutine:       /////////////
//////////////////////////////////////////////////////////////

void calculate_a(int ndata, float x[], float y[], float fl[], float flx[], float fl_mod[], int nrows, float *fl_border, float fit_x0, float fit_y0, float *af, float *eraf)
{
  //////////////////////calculate the "a" factor//////////////////
  printf("\n......calculating: a (asymmetry factor)......\n");
  float sum_a=0.,sum_fl=0.;
  int i,j,flag_x,flag_y,flag_i;
  
  for(i=1;i<=ndata;i++)
    {
      if(fl_mod[i]>*fl_border)
	{
	  flag_x=(int)(2*fit_x0-x[i]);
	  flag_y=(int)(2*fit_y0-y[i]);
	  flag_i=(flag_y-1)*nrows+flag_x;
	  if(flx[flag_i]>0 && fl[i]>0)
	    {
	      sum_a=sum_a+pow(fl[i]-flx[flag_i],2.);
	      sum_fl=sum_fl+fl[i]*fl[i];
	    }
	}
    }
  
  if(sum_fl>0.)
    {*af=log10(sum_a/sum_fl);}
  else
    {*af=0.;}

  ////////////////////////error of a////////////////////////////
  int m,k=10,num_rand,*pt,Ta,Tb;
  float *asy,sumasy=0.,erasy=0.,no_data=0.;

  pt=ivector(1,ndata); asy=vector(1,k);
  srand((unsigned)time(NULL));
  num_rand=floor(ndata/2.);

  for(m=1;m<=k;m++)
    {
      sum_a=0.; sum_fl=0.;
      for(i=1;i<=num_rand;i++)
	{
	  Tb=rand()%(ndata-i)+i;
	  Ta=pt[i];
	  pt[i]=pt[Tb];
	  pt[Tb]=Ta;
	  if(fl_mod[i]>*fl_border)
	    {
	      flag_x=(int)(2*fit_x0-x[i]);
	      flag_y=(int)(2*fit_y0-y[i]);
	      flag_i=(flag_y-1)*nrows+flag_x;
	      if(flx[flag_i]>0 && fl[i]>0)
		{
		  sum_a=sum_a+pow(fl[i]-flx[flag_i],2.);
		  sum_fl=sum_fl+fl[i]*fl[i];
		}
	    }
	}
      if(sum_fl==0.)
	{no_data=no_data+1.;}
      else
	{
	  asy[m]=sum_a*100./sum_fl;
	  sumasy=sumasy+asy[m];
	}
    }
  sumasy=sumasy/(k-no_data);
  for(i=1;i<=k;i++)
    {
      erasy=erasy+pow(asy[i]-sumasy,2.);
    }
  erasy=sqrt(erasy/k);
  erasy=erasy/((*af)*2.3);
  if(erasy<0.01)
    {erasy=0.01;}
  //printf("%f %f %f\n",asy,erasy,ndt1);
  *eraf=erasy;
  if(no_data==k)
    {*eraf=0.;}
}

void calculate_p(float z, int ndata, float x[], float y[], float fl[], float *mean, float fit_x0, float fit_y0, float scale500, float *pp, float *erpp)
{
  //////////////////////calculate the "P3/P0"////////////////////////
  printf("\n......calculating: p (power ratio)......\n");

  if(z>5)
    {
      printf("No redshift available! \"p\" can NOT be calculated!\n");
      *pp=0.; *erpp=0.;
    }
  else
    {
      int i,j;
      float r,*phi,*phc,*phs,*rr,*coa,*cob,sum_coa=0.,sum_cob=0.,sum_fl=0.,p3,p0;
      phi=vector(1,ndata); phc=vector(1,ndata); phs=vector(1,ndata);
      rr=vector(1,ndata); coa=vector(1,ndata); cob=vector(1,ndata);
  
      for(i=1;i<=ndata;i++)
	{
	  r=sqrt(pow(x[i]-fit_x0,2.)+pow(y[i]-fit_y0,2.));
	  if(r<scale500)
	    {
	      phi[i]=atan2(y[i]-fit_y0,x[i]-fit_x0);
	      phc[i]=cos(3.*phi[i]);
	      phs[i]=sin(3.*phi[i]);
	      rr[i]=(fl[i]-*mean)*pow(pow(x[i]-fit_x0,2)+pow(y[i]-fit_y0,2),1.5);
	      coa[i]=rr[i]*phc[i];
	      cob[i]=rr[i]*phs[i];
	      sum_coa=sum_coa+coa[i];
	      sum_cob=sum_cob+cob[i];
	      sum_fl=sum_fl+fl[i];
	    }
	}
      p3=(pow(sum_coa,2)+pow(sum_cob,2))/(18.*pow(scale500,6));
      p0=pow(sum_fl*log(scale500),2);
      *pp=log10(p3/p0);

  ////////////////////////error of p////////////////////////////
      int k=10,num_rand,*pt,Ta,Tb;
      float *p3p0,sump3p0=0.,erp3p0=0.;

      pt=ivector(1,ndata); p3p0=vector(1,k); 
      srand((unsigned)time(NULL));
  
      num_rand=floor(ndata/2.);

      for(j=1;j<=k;j++)
	{
	  sum_coa=0.; sum_cob=0.; sum_fl=0.; p3=0.; p0=0.;
	  for(i=1;i<=ndata;i++)
	    {
	      pt[i]=i;
	    }
	  for(i=1;i<=num_rand;i++)
	    {
	      Tb=rand()%(ndata-i)+i;
	      Ta=pt[i];
	      pt[i]=pt[Tb];
	      pt[Tb]=Ta;
	      r=sqrt(pow(x[pt[i]]-fit_x0,2.)+pow(y[pt[i]]-fit_y0,2.));
	      if(r<scale500)
		{
		  phi[i]=atan2(y[pt[i]]-fit_y0,x[pt[i]]-fit_x0);
		  phc[i]=cos(3.*phi[i]);
		  phs[i]=sin(3.*phi[i]);
		  rr[i]=(fl[pt[i]]-*mean)*pow(pow(x[pt[i]]-fit_x0,2)+pow(y[pt[i]]-fit_y0,2),1.5);
		  coa[i]=rr[i]*phc[i];
		  cob[i]=rr[i]*phs[i];
		  sum_coa=sum_coa+coa[i];
		  sum_cob=sum_cob+cob[i];
		  sum_fl=sum_fl+fl[pt[i]];
		}
	    }
	  p3=(pow(sum_coa,2)+pow(sum_cob,2))/(18.*pow(scale500,6));
	  p0=pow(sum_fl*log(scale500),2);
	  p3p0[j]=log10(p3/p0);
	  sump3p0=sump3p0+p3p0[j];
	}
      sump3p0=sump3p0/k;
      for(i=1;i<=k;i++)
	{
	  erp3p0=erp3p0+pow(p3p0[i]-sump3p0,2.);
	}
      erp3p0=sqrt(erp3p0/k);
      if(erp3p0<0.01)
	{erp3p0=0.01;}
      *erpp=erp3p0;
    }
}

void calculate_c(float z, int ndata, float x[], float y[], float fl[], float *mean, float *rms, float fit_x0, float fit_y0, float scale500, float *ci, float *erci)
{
  
  /////////////////////calculate the "c" index///////////////////////
  printf("\n......calculating: c (concentration index)......\n");
  if(z>5)
    {
      printf("No redshift available! \"c\" can NOT be calculated!\n");
      *ci=0.; *erci=0.;
    }
  else
    {
      int i,j,*cnt_i,annu,ann_flag=0;
      float sum_cntr=0.,sum_tot=0.,r,r_flag,*sumfl,*meanfl;

      cnt_i=ivector(1,20); sumfl=vector(1,20); meanfl=vector(1,20);
     
      for(i=1;i<=20;i++)
	{
	  sumfl[i]=0.; cnt_i[i]=0;
	}
      for(i=1;i<=ndata;i++)
	{
	  if(x[i]==fit_x0 && y[i]==fit_y0)
	    {r=0.;}
	  else
	    {r=sqrt(pow(x[i]-fit_x0,2.)+pow(y[i]-fit_y0,2.));}
	  if(r<scale500 && fl[i]>(*mean-*rms*3))
	    {
	      annu=floor(r/(scale500/20.))+1;
	      cnt_i[annu]++;
	      sum_tot=sum_tot+fl[i]-*mean;
	    }
	  if(r<floor(scale500/5.)+1 && fl[i]>(*mean-*rms*3))
	    {
	      sum_cntr=sum_cntr+fl[i]-*mean;
	    }
	}
      if(cnt_i[20]==0.)
	{
	  ann_flag=1;
	  printf("No data in %dth annu!\n\"c\" can NOT be calculated!\n",i);
	}
      if(ann_flag<1)
	{
	  *ci=log10(sum_cntr/sum_tot);
	  //printf("%f %f %f\n",sum1,sum2,ci);
	}
      else
	{*ci=0.00;}
      
      //////////////////error of c///////////////////////////////
      int k=10,num_rand,*pt,Ta,Tb;
      float *cndx,sumcndx,erc=0.;

      cndx=vector(1,k); pt=ivector(1,ndata);
      srand((unsigned)time(NULL));

      num_rand=floor(ndata/2.);
      sumcndx=0.; 

      for(j=1;j<=k;j++)
	{
	  sum_cntr=0.; sum_tot=0.;
	  for(i=1;i<=20;i++)
	    {
	      sumfl[i]=0.; cnt_i[i]=0;
	    }
	  for(i=1;i<=ndata;i++)
	    {
	      pt[i]=i;
	    }
	  for(i=1;i<=num_rand;i++)
	    {
	      Tb=rand()%(ndata-i)+i;
	      Ta=pt[i];
	      pt[i]=pt[Tb];
	      pt[Tb]=Ta;
	      r=sqrt(pow(x[pt[i]]-fit_x0,2.)+pow(y[pt[i]]-fit_y0,2.));
	      if(r<scale500 && fl[pt[i]]>(*mean-*rms))
		{
		  annu=floor(r/(scale500/20.))+1;
		  cnt_i[annu]++;
		  sum_tot=sum_tot+fl[pt[i]]-*mean;
		}
	      if(r<floor(scale500/5.)+1 && fl[pt[i]]>(*mean-*rms))
		{
		  sum_cntr=sum_cntr+fl[pt[i]]-*mean;
		}
	    }
	  cndx[j]=log10(sum_cntr/sum_tot);
	  //printf("%.2f\n",cndx[j]);
	  sumcndx=sumcndx+cndx[j];
	}
      sumcndx=sumcndx/k;
      for(i=1;i<=k;i++)
	{
	  erc=erc+pow(cndx[i]-sumcndx,2.);
	}
      *erci=sqrt(erc/k);
      if(*erci<0.01)
	{*erci=0.01;}
      if(ann_flag==1)
	{*erci=0.00;}
    }
}

void calculate_w(float z, int ndata, float x[], float y[], float fl[], float *mean, float fit_x0, float fit_y0, float scale500, float *peak_x, float *peak_y, float *maxfl, float *w, float *erw)
{
  //////////////////////calculate the "w" factor/////////////////////

  printf("\n......calculating: w (centroid shift)......\n");
  if(z>5)
    {
      printf("No redshift available! \"w\" can NOT be calculated!\n");
      *w=0.; *erw=0.;
    }
  else
    {
      float *a,**covar,**alpha,chisq[1],alamda[1];
      int i,j,*ia,ma,na;
 
      void mrqmin2D2(float x1[], float x2[], float y[], int ndata, float a[], int ia[], int ma, float **covar, float **alpha, float *chisq, void funcs2D(float, float, float [], float *, float [], int), float *alamda);

      ma=8; na=ma;
      a=vector(1,ma); ia=ivector(1,ma);
      covar=matrix(1,ma,1,ma); alpha=matrix(1,ma,1,ma);
  
      //initialize fitting parameters --> circular model: fixed ellipticity and position angle
      ia[1]=1; ia[2]=1; ia[3]=1; ia[4]=0;
      ia[5]=0; ia[6]=1; ia[7]=1; ia[8]=1;

      float chisq_tmp,temp,*x_apt,*y_apt,*fl_apt,a_apt[8],*fitx,*fity,*delta,meandelta,sumdelta,*omg,meanomg,sumx,sumy,sigx,sigy;
      int loop,mm,jj,flag,n_apt=0; 
      x_apt=vector(1,ndata); y_apt=vector(1,ndata); fl_apt=vector(1,ndata);
      fitx=vector(1,20); fity=vector(1,20); delta=vector(1,20);
      omg=vector(1,10);
  
      meandelta=0.; sumdelta=0.; flag=0; meanomg=0.;
      sumx=0.; sumy=0.; sigx=0.; sigy=0.;
  
      for(mm=1;mm<=20;mm++)
	{
	  n_apt=0; 
	  memset(x_apt,0,ndata);
	  memset(y_apt,0,ndata);
	  memset(fl_apt,0,ndata);
	  for(i=1;i<=ndata;i++)
	    {
	      if(sqrt(pow(x[i]-*peak_x,2.)+pow(y[i]-*peak_y,2.))<mm*scale500/20.)
		{
		  n_apt++;
		  x_apt[n_apt]=x[i]; y_apt[n_apt]=y[i]; fl_apt[n_apt]=fl[i];
		}
	    }

	  a[1]=*maxfl; a[2]=mm*scale500/10.; a[3]=0.8; a[4]=0.;// try 0.8
	  a[5]=0.; a[6]=*peak_x; a[7]=*peak_y; a[8]=0.;
	  if(n_apt>=20)//fitting apture should have at least 20 pixels
	    {
	      chisq_tmp=100000.; loop=0;
	      for(i=1;i<=150;i++)
		{
		  loop++;
		  *alamda=-5.;
		  mrqmin2D2(x_apt,y_apt,fl_apt,n_apt,a,ia,ma,covar,alpha,chisq,funcs2D,alamda);
		  if(*chisq<chisq_tmp*0.99 || loop<=20)
		    {
		      chisq_tmp=*chisq;
		    }
		  else if(a[3]>0.5 && a[8]>0.)
		    {
		      //the chi-square is stable and coefficients in reasonlabe ranges, adopted
		      break;
		    }
		  else
		    {chisq_tmp=chisq_tmp*1.2;}//re-fitting with below new-initialization
		  if(a[3]<=0.5) {a[3]=0.8;}
		}
	      temp=*chisq;

	      for(i=1;i<=ma;i++)
		{
		  a_apt[i-1]=a[i];
		}

	      a[1]=*maxfl; a[2]=mm*scale500/10.; a[3]=1.2; a[4]=0.; //try 1.2
	      a[5]=0.; a[6]=*peak_x; a[7]=*peak_y; a[8]=0.;
	      chisq_tmp=100000.; loop=0;
    
	      for(i=1;i<=150;i++)
		{
		  loop++;
		  *alamda=-5.;
		  mrqmin2D2(x_apt,y_apt,fl_apt,n_apt,a,ia,ma,covar,alpha,chisq,funcs2D,alamda);
		  if(*chisq<chisq_tmp*0.99 || loop<=20)
		    {
		      chisq_tmp=*chisq;
		    }
		  else if(a[3]>0.5 && a[8]>0.)
		    {
		      break;
		    }
		  else
		    {chisq_tmp=chisq_tmp*1.2;}
		  if(a[3]<=0.5) {a[3]=0.8;}
		}

	      if(*chisq<temp)
		{fitx[mm]=a[6]; fity[mm]=a[7];}
	      else
		{fitx[mm]=a_apt[5]; fity[mm]=a_apt[6];}
	    }
	  else
	    {fitx[mm]=*peak_x; fity[mm]=*peak_y;}
	  //printf("%d %f %f %f %f\n",s1,fitx[mm],fity[mm],maxx,maxy);
	  sumx=sumx+fitx[mm]; sumy=sumy+fity[mm];
	}
      sumx=sumx/20.; sumy=sumy/20.;
      for(mm=1;mm<=20;mm++)
	{
	  sigx=sigx+pow(fitx[mm]-sumx,2.);
	  sigy=sigy+pow(fity[mm]-sumy,2.);
	}
      sigx=sqrt(sigx/20.); sigy=sqrt(sigy/20.);
      if(sigx<0.5) {sigx=0.5;}
      if(sigy<0.5) {sigy=0.5;}
      for(mm=1;mm<=20;mm++)
	{//delete error fitting psizeults ---> very large departure
	  if(fabs(fitx[mm]-*peak_x)<5.*sigx && fabs(fity[mm]-*peak_y)<5.*sigy)//no-large departure to the peak
	    {
	      flag++;
	      delta[flag]=sqrt(pow(fitx[mm]-*peak_x,2.)+pow(fity[mm]-*peak_y,2.));
	      meandelta=meandelta+delta[flag];
	    }
	  else if(fabs(fitx[mm]-*peak_x)<1.5*fabs(*peak_x-fit_x0) && fabs(fity[mm]-*peak_y)<1.5*fabs(*peak_y-fit_y0))//no-large departure to the fitted center of the whole region
	    {
	      flag++;
	      delta[flag]=sqrt(pow(fitx[mm]-*peak_x,2.)+pow(fity[mm]-*peak_y,2.));
	      meandelta=meandelta+delta[flag];
	    }
	}
      //printf("\n%f %f\n%f %f\n",x0,y0,maxx,maxy);
      meandelta=meandelta/20.;
      for(i=1;i<=flag;i++)
	{
	  sumdelta=sumdelta+pow(delta[i]-meandelta,2.);
	}
      *w=log10(sqrt(sumdelta/(flag-1.))/scale500);
      //printf("%d %f\n",flag,w);
  
      ////////////////////the error of w/////////////////////////////////
      int k=10,*pt,Ta,Tb,num_rand;
      pt=ivector(1,ndata);
      srand((unsigned)time(NULL));
      num_rand=floor(ndata/2.);

      for(jj=1;jj<=k;jj++)
	{
	  meandelta=0.; sumdelta=0.; flag=0;
	  for(i=1;i<=ndata;i++)
	    {
	      pt[i]=i;
	    }
	  for(i=1;i<=num_rand;i++)
	    {
	      Tb=rand()%(ndata-i)+i;
	      Ta=pt[i];
	      pt[i]=pt[Tb];
	      pt[Tb]=Ta;
	    }
	  for(mm=1;mm<=20;mm++)
	    {
	      memset(x_apt,0,ndata);
	      memset(y_apt,0,ndata);
	      memset(fl_apt,0,ndata);
	      n_apt=0; 
	      for(i=1;i<=num_rand;i++)
		{
		  if(sqrt(pow(x[pt[i]]-*peak_x,2.)+pow(y[pt[i]]-*peak_y,2.))<mm*scale500/20.)
		    {
		      n_apt++;
		      x_apt[n_apt]=x[pt[i]]; 
		      y_apt[n_apt]=y[pt[i]]; 
		      fl_apt[n_apt]=fl[pt[i]];
		    }
		}

	      a[1]=*maxfl; a[2]=mm*scale500/10.; a[3]=0.8; a[4]=0.;// try 0.8
	      a[5]=0.; a[6]=*peak_x; a[7]=*peak_y; a[8]=0.;
	      if(n_apt>=20)
		{
		  chisq_tmp=100000.; loop=0;
		  for(i=1;i<=150;i++)
		    {
		      loop++;
		      *alamda=-5.;
		      mrqmin2D2(x_apt,y_apt,fl_apt,n_apt,a,ia,ma,covar,alpha,chisq,funcs2D,alamda);
		      if(*chisq<chisq_tmp*0.99 || loop<=20)
			{
			  chisq_tmp=*chisq;
			}
		      else if(a[3]>0.5 && a[8]>0.)
			{
			  break;
			}
		      else
			{chisq_tmp=chisq_tmp*1.2;}
		      if(a[3]<=0.5) {a[3]=0.8;}
		    }
		  temp=*chisq;

		  for(i=1;i<=ma;i++)
		    {
		      a_apt[i-1]=a[i];
		    }

		  a[1]=*maxfl; a[2]=mm*scale500/10.; a[3]=1.2; a[4]=0.; //try 1.2
		  a[5]=0.; a[6]=*peak_x; a[7]=*peak_y; a[8]=0.;
		  chisq_tmp=100000.; loop=0;
    
		  for(i=1;i<=150;i++)
		    {
		      loop++;
		      *alamda=-5.;
		      mrqmin2D2(x_apt,y_apt,fl_apt,n_apt,a,ia,ma,covar,alpha,chisq,funcs2D,alamda);
		      if(*chisq<chisq_tmp*0.99 || loop<=20)
			{
			  chisq_tmp=*chisq;
			}
		      else if(a[3]>0.5 && a[8]>0.)
			{
			  break;
			}
		      else
			{chisq_tmp=chisq_tmp*1.2;}
		      if(a[3]<=0.5) {a[3]=0.8;}
		    }

		  if(*chisq<temp)
		    {fitx[mm]=a[6]; fity[mm]=a[7];}
		  else
		    {fitx[mm]=a_apt[5]; fity[mm]=a_apt[6];}
		}
	      else
		{fitx[mm]=*peak_x; fity[mm]=*peak_y;}
	      if(fabs(fitx[mm]-*peak_x)<5.*sigx && fabs(fity[mm]-*peak_y)<5.*sigy)
		{
		  flag++;
		  delta[flag]=sqrt(pow(fitx[mm]-*peak_x,2.)+pow(fity[mm]-*peak_y,2.));
		  meandelta=meandelta+delta[flag];
		}
	      else if(fabs(fitx[mm]-*peak_x)<1.5*fabs(*peak_x-fit_x0) && fabs(fity[mm]-*peak_y)<1.5*fabs(*peak_y-fit_y0))
		{
		  flag++;
		  delta[flag]=sqrt(pow(fitx[mm]-*peak_x,2.)+pow(fity[mm]-*peak_y,2.));
		  meandelta=meandelta+delta[flag];
		}
	    }  
	  meandelta=meandelta/(1.*flag);
	  //printf("\n%f %f\n%f %f\n",x0,y0,maxx,maxy);
	  for(i=1;i<=flag;i++)
	    {
	      sumdelta=sumdelta+pow(delta[i]-meandelta,2.);
	    }
	  omg[jj]=log10(sqrt(sumdelta/(flag-1.))/scale500);
	  //printf("%d %f\n",flag,omg[jj]);
	  meanomg=meanomg+omg[jj];
	}
      meanomg=meanomg/k;
  
      float eromg; eromg=0.;
      for(i=1;i<=k;i++)
	{
	  eromg=eromg+pow(omg[i]-meanomg,2.);
	}
      eromg=sqrt(eromg/k);
      if(eromg<0.01)
	{eromg=0.01;}
      *erw=eromg;
    }
}

  ////            plot     observation     image          ////
void plotobs(char name[], float z, float psize, float *cntra, float *cntdec, float cntx0, float cnty0, float fit_x0, float fit_y0, float *minx, float *maxx, float *miny, float *maxy, float *minfl, float *maxfl, float *mean, float *rms, float scale500, float scale30, float *ci, float *w, float *pp, float *dlt)
{
  int i,RAH,RAM,decd,decm;
  float DecMin,DecMax,deltaDecMark,RAmin,RAmax,deltaRAmark,RAmark[4],RAS,DecMark[4],RAmin1,RAmax1,deltaRAmark1,RAmark1[4],ras,decs,*px,*px1,*py; 
  char filename1[50],filename2[50],filename3[50],filename4[50],sign[5],**ticx,**ticy; 

  px=vector(1,4); px1=vector(1,4); py=vector(1,4);
  ticx=mcvector(11,30); ticy=mcvector(11,30);

  sprintf(filename1,"%s_image.eps",name);
  sprintf(filename2,"%s_obj.dat",name);
  sprintf(filename3,"%s_objcntr.dat",name);
  sprintf(filename4,"%s_modcntrout.dat",name);

  DecMin = 3600.*(*cntdec)-(cnty0-*miny)*psize;
  DecMax = 3600.*(*cntdec)+(*maxy-cnty0)*psize;
  deltaDecMark = floor((DecMax-DecMin)/60.*0.5); // in arcmin

  for(i=1;i<=4;i++)
    {
      DecMark[i]=floor(DecMin/60)+1+(i-1)*deltaDecMark ;   // in arcmin
      py[i]=(DecMark[i]*60-DecMin)/psize+*miny; 
      
      if(DecMark[i] >=0 ) 
	{strcpy(sign,"+");}
      else 
	{
	  strcpy(sign,"-"); 
	  DecMark[i]=abs(DecMark[i]);
	}
      
      decd=floor(DecMark[i]/60);
      decm=floor(DecMark[i]-(decd*60));
      decs=0;
      sprintf(ticy[i],"%1s%02d:%02d:%02.0f",sign,decd,decm,decs);
    }

  RAmin=3600.*(*cntra)-(*maxx-cntx0)*psize/cos(DecMin*3.14159/180./3600.); //in arcsec
  RAmax=3600.*(*cntra)+(cntx0-*minx)*psize/cos(DecMin*3.14159/180./3600.); //in arcsec
  deltaRAmark=floor((RAmax-RAmin)/2.*1.2/75)*75;                         //in arcsec
  
  for(i=1;i<=4;i++)  // for two marks in RA ****, find RAmark[1] and RAmark
    {
      RAmark[i]=floor(RAmin/15)+1+(i-1)*deltaRAmark/15 ; // in hour-sec
      RAmark[i]=(floor(RAmark[i]/5)+1)*5;
      px[i]=*maxx-(RAmark[i]*15-RAmin)/(psize/cos(DecMin*3.14159/180./3600.));

      RAH=floor(RAmark[i]/3600);        // in hour-H
      RAM=floor(RAmark[i]/60-(RAH*60)); // in hour-M
      RAS=RAmark[i]-RAH*3600-RAM*60;    // in hour-S
      sprintf(ticx[i],"%02d:%02d:%02.0f",RAH,RAM,RAS);
    }

  RAmin1=3600.*(*cntra)-(*maxx-cntx0)*psize/cos(DecMax*3.14159/180./3600.); //in arcsec
  RAmax1=3600.*(*cntra)+(cntx0-*minx)*psize/cos(DecMax*3.14159/180./3600.); //in arcsec
  deltaRAmark1=floor((RAmax1-RAmin1)/2.*1.2/75)*75;                       //in arcsec
  
  for(i=1;i<=4;i++)  
    {
      RAmark1[i]=floor(RAmin1/15)+1+(i-1)*deltaRAmark1/15 ; // in hour-sec
      RAmark1[i]=(floor(RAmark1[i]/5)+1)*5;
      px1[i]=*maxx-(RAmark1[i]*15-RAmin1)/(psize/cos(DecMax*3.14159/180./3600.));
    }
  FILE * command=popen("gnuplot","w");  
  fprintf(command, "set terminal post enhanced color size 6,5 solid \"Times-Roman\" 16\n");
  fprintf(command, "set output \"%s\"\n",filename1);
  if(z>5)
    {fprintf(command, "set title \"%s (no z): XMM-Newton image\" offset 0,-0.8 font \"Helvetica,14\" \n",name);}
  else
    {fprintf(command, "set title \"%s (z=%.4f): XMM-Newton image\" offset 0,-0.8 font \"Helvetica,14\"\n",name,z);}
  fprintf(command, "set label 1 \"Contours: <S_{bg}>+5{/Symbol \\264}2^n{/Symbol \\163}, n = 0, 1, 2, 3, ...\" at graph 0.5,1.06 center front font \"Helvetica,14\"\n");
  fprintf(command, "set label 2 \"\" at %f,%f center point pt 1 ps 1.2 lw 1 lc rgbcolor \"white\" front\n",fit_x0,fit_y0);
  if(z<5) {
    if(z>0.03) {
      fprintf(command, "set label 3 \"500 kpc\" at graph 0.97,0.07 right front font \"Helvetica,14\" tc rgbcolor \"red\" \n");
      fprintf(command, "set arrow 1 nohead from %f,%f to %f,%f lw 0.5 lc rgbcolor \"red\" front\n",*maxx-0.03*(*maxx-*minx),*miny+0.03*(*maxy-*miny),*maxx-0.03*(*maxx-*minx)-scale500,*miny+0.03*(*maxy-*miny));
      fprintf(command, "set arrow 2 nohead from %f,%f to %f,%f lw 0.5 lc rgbcolor \"red\" front\n",*maxx-0.03*(*maxx-*minx),*miny+0.01*(*maxy-*miny),*maxx-0.03*(*maxx-*minx),*miny+0.05*(*maxy-*miny));
      fprintf(command, "set arrow 3 nohead from %f,%f to %f,%f lw 0.5 lc rgbcolor \"red\" front\n",*maxx-0.03*(*maxx-*minx)-scale500,*miny+0.01*(*maxy-*miny),*maxx-0.03*(*maxx-*minx)-scale500,*miny+0.05*(*maxy-*miny));
      fprintf(command, "set object 1 circle at %f,%f size %f front lw 0.5 fc rgbcolor \"red\" fs solid\n",*minx+(*maxx-*minx)/40.+scale30/2.,*miny+(*maxy-*miny)/40.+scale30/2.,scale30/2.);}
    else { scale500=scale500/5; scale30=scale30/3;
      fprintf(command, "set label 3 \"100 kpc\" at graph 0.97,0.07 right front font \"Helvetica,14\" tc rgbcolor \"red\" \n");
      fprintf(command, "set arrow 1 nohead from %f,%f to %f,%f lw 0.5 lc rgbcolor \"red\" front\n",*maxx-0.03*(*maxx-*minx),*miny+0.03*(*maxy-*miny),*maxx-0.03*(*maxx-*minx)-scale500,*miny+0.03*(*maxy-*miny));
      fprintf(command, "set arrow 2 nohead from %f,%f to %f,%f lw 0.5 lc rgbcolor \"red\" front\n",*maxx-0.03*(*maxx-*minx),*miny+0.01*(*maxy-*miny),*maxx-0.03*(*maxx-*minx),*miny+0.05*(*maxy-*miny));
      fprintf(command, "set arrow 3 nohead from %f,%f to %f,%f lw 0.5 lc rgbcolor \"red\" front\n",*maxx-0.03*(*maxx-*minx)-scale500,*miny+0.01*(*maxy-*miny),*maxx-0.03*(*maxx-*minx)-scale500,*miny+0.05*(*maxy-*miny));
      fprintf(command, "set object 1 circle at %f,%f size %f front lw 0.5 fc rgbcolor \"red\" fs solid\n",*minx+(*maxx-*minx)/40.+scale30/2.,*miny+(*maxy-*miny)/40.+scale30/2.,scale30/2.);}}
  fprintf(command, "set label 4 \"log(c)=%.2f\" at graph 0.02,0.96 left front font \"Helvetica-Bold,13\" textcolor \"black\"\n",*ci);
  fprintf(command, "set label 5 \"log({/Symbol \\167})=%.2f\" at graph 0.02,0.90 left front font \"Helvetica-Bold,13\" textcolor \"black\"\n",*w);
  fprintf(command, "set label 6 \"log(P_3/P_0)=%.2f\" at graph 0.02,0.84 left front font \"Helvetica-Bold,13\" textcolor \"black\"\n",*pp);
  fprintf(command, "set label 7 \"{/Symbol \\144}=%.2f\" at graph 0.02,0.78 left front font \"Helvetica-Bold,13\" textcolor \"black\"\n",*dlt);
  fprintf(command, "set contour base\n");
  fprintf(command, "unset surface\n");
  fprintf(command, "set size square\n");
  fprintf(command, "set xlabel \"RA (J2000)\" offset 0,1 font \"Helvetica,14\"\n");
  fprintf(command, "set ylabel \"DEC (J2000)\" offset 1.5,0 font \"Helvetica,14\"\n");
  fprintf(command, "set xrange [%f:%f]\n",*minx,*maxx);
  fprintf(command, "set yrange [%f:%f]\n",*miny,*maxy);
  fprintf(command, "set cbrange [1:%f]\n",*maxfl);
  fprintf(command, "set xtics (\"%s\" %f, \"%s\" %f, \"%s\" %f, \"%s\" %f) out nomirror offset 0,0.5 font \"Helvetica,14\"\n",ticx[1],px[1],ticx[2],px[2],ticx[3],px[3],ticx[4],px[4]);
  fprintf(command, "set x2tics (\"\" %f, \"\" %f, \"\" %f) out\n",px1[1],px1[2],px1[3]);
  fprintf(command, "set ytics (\"%s\" %f, \"%s\" %f, \"%s\" %f, \"%s\" %f) out mirror rotate by 90 center offset -0.5,0 font \"Helvetica,14\"\n",ticy[1],py[1],ticy[2],py[2],ticy[3],py[3],ticy[4],py[4]);
  fprintf(command, "set cblabel \"Brightness [cnt s^{-1} deg^{-2}]\"offset 0.2,0 font \"Helvetica,14\"\n");
  fprintf(command, "set cbtics (\"1\" 1, \"10\" 10, \"10^{2}\" 100, \"10^3\" 1000) offset -0.7,0 rotate by 90  font \"Helvetica,14\"\n");
  fprintf(command, "set logscale cb\n");
  fprintf(command, "set cntrparam levels discrete %f+%f*5.,%f+%f*10.,%f+%f*20.,%f+%f*40.,%f+%f*80.,%f+%f*160.,%f+%f*320.,%f+%f*640.,%f+%f*1280.,%f+%f*2560.,%f+%f*5120.,%f+%f*10240.,%f+%f*20480.\n",*mean,*rms,*mean,*rms,*mean,*rms,*mean,*rms,*mean,*rms,*mean,*rms,*mean,*rms,*mean,*rms,*mean,*rms,*mean,*rms,*mean,*rms,*mean,*rms,*mean,*rms);
  fprintf(command, "set table '%s'\n",filename3);
  fprintf(command, "splot '%s'\n",filename2);
  fprintf(command, "unset table\n");
  fprintf(command, "set palette rgbformulae 22,13,-31\n");
  fprintf(command, "plot '%s' w image notitle, '%s' w lines lw 3 lc rgbcolor \"white\" lt 1 notitle, '%s' w lines lw 1 lc rgbcolor \"black\" notitle, '%s' w lines lw 3 lt 1 lc rgbcolor \"white\" notitle \n",filename2,filename3,filename3,filename4);
  fprintf(command, "set output\n");
  fflush(command);
  fclose(command);
}

  ////            plot     fitting      model     image           ////
void plotmod(char name[], float psize, float *cntra, float *cntdec, float cntx0, float cnty0, float ellip, float fit_x0, float fit_y0, float *minx, float *maxx, float *miny, float *maxy, float *minfl, float *maxfl, float *mean, float *rms, float *fl_border)
{
  int i,RAH,RAM,decd,decm;
  float DecMin,DecMax,deltaDecMark,RAmin,RAmax,deltaRAmark,RAmark[4],RAS,DecMark[4],RAmin1,RAmax1,deltaRAmark1,RAmark1[4],ras,decs,*px,*px1,*py; 
  char filename1[50],filename2[50],filename3[50],filename4[50],sign[5],**ticx,**ticy; 

  px=vector(1,4); px1=vector(1,4); py=vector(1,4);
  ticx=mcvector(11,30); ticy=mcvector(11,30);

  sprintf(filename1,"%s_model.eps",name);
  sprintf(filename2,"%s_mod.dat",name);
  sprintf(filename3,"%s_modcntr.dat",name);
  sprintf(filename4,"%s_modcntrout.dat",name);

  DecMin = 3600.*(*cntdec)-(cnty0-*miny)*psize;
  DecMax = 3600.*(*cntdec)+(*maxy-cnty0)*psize;
  deltaDecMark = floor((DecMax-DecMin)/60. *0.6); // in arcmin

  for(i=1;i<=4;i++)
    {
      DecMark[i]=floor(DecMin/60)+1+(i-1)*deltaDecMark ;   // in arcmin
      py[i]=(DecMark[i]*60-DecMin)/psize+*miny; 
      
      if(DecMark[i] >=0 ) 
	{strcpy(sign,"+");}
      else 
	{
	  strcpy(sign,"-"); 
	  DecMark[i]=abs(DecMark[i]);
	}
      
      decd=floor(DecMark[i]/60);
      decm=floor(DecMark[i]-(decd*60));
      decs=0;
      sprintf(ticy[i],"%1s%02d:%02d:%02.0f",sign,decd,decm,decs);
    }

  RAmin=3600.*(*cntra)-(*maxx-cntx0)*psize/cos(DecMin*3.14159/180./3600.); //in arcsec
  RAmax=3600.*(*cntra)+(cntx0-*minx)*psize/cos(DecMin*3.14159/180./3600.); //in arcsec
  deltaRAmark=floor((RAmax-RAmin)/2.*1.2/75)*75;                         //in arcsec
  
  for(i=1;i<=4;i++)  // for two marks in RA ****, find RAmark[1] and RAmark
    {
      RAmark[i]=floor(RAmin/15)+1+(i-1)*deltaRAmark/15 ; // in hour-sec
      RAmark[i]=(floor(RAmark[i]/5)+1)*5;
      px[i]=*maxx-(RAmark[i]*15-RAmin)/(psize/cos(DecMin*3.14159/180./3600.));

      RAH=floor(RAmark[i]/3600);        // in hour-H
      RAM=floor(RAmark[i]/60-(RAH*60)); // in hour-M
      RAS=RAmark[i]-RAH*3600-RAM*60;    // in hour-S
      sprintf(ticx[i],"%02d:%02d:%02.0f",RAH,RAM,RAS);
    }

  RAmin1=3600.*(*cntra)-(*maxx-cntx0)*psize/cos(DecMax*3.14159/180./3600.); //in arcsec
  RAmax1=3600.*(*cntra)+(cntx0-*minx)*psize/cos(DecMax*3.14159/180./3600.); //in arcsec
  deltaRAmark1=floor((RAmax1-RAmin1)/2.*1.2/75)*75;                       //in arcsec
  
  for(i=1;i<=4;i++)  
    {
      RAmark1[i]=floor(RAmin1/15)+1+(i-1)*deltaRAmark1/15 ; // in hour-sec
      RAmark1[i]=(floor(RAmark1[i]/5)+1)*5;
      px1[i]=*maxx-(RAmark1[i]*15-RAmin1)/(psize/cos(DecMax*3.14159/180./3600.));
    }
  FILE * command=popen("gnuplot","w");  
  fprintf(command, "set terminal post enhanced color size 6,5 solid \"Times-Roman\" 16\n");
  fprintf(command, "set output \"%s\"\n",filename1);
  fprintf(command, "set title \"%s: Fitting model\" offset 0,-0.8 font \"Helvetica,14\" \n",name);
  fprintf(command, "set contour base\n");
  fprintf(command, "unset surface\n");
  fprintf(command, "set size square\n");
  fprintf(command, "set xlabel \"RA (J2000)\" offset 0,1 font \"Helvetica,14\"\n");
  fprintf(command, "set ylabel \"DEC (J2000)\" offset 1.5,0 font \"Helvetica,14\"\n");
  fprintf(command, "set label 1 \"Contours: <S_{bg}>+5{/Symbol \\264}2^n{/Symbol \\163}, n = 0, 1, 2, 3, ...\" at graph 0.5,1.06 center front font \"Helvetica,14\"\n");
  fprintf(command, "set label 2 \"\" at %f,%f center point pt 1 ps 1.2 lw 1 lc rgbcolor \"white\" front\n",fit_x0,fit_y0);

  fprintf(command, "set label 3 \" e = %.2f\" at graph 0.97,0.07 right front font \"Helvetica,14\" tc rgbcolor \"red\" \n",ellip);

  fprintf(command, "set xrange [%f:%f]\n",*minx,*maxx);
  fprintf(command, "set yrange [%f:%f]\n",*miny,*maxy);
  fprintf(command, "set cbrange [1:%f]\n",*maxfl);
  fprintf(command, "set xtics (\"%s\" %f, \"%s\" %f, \"%s\" %f, \"%s\" %f) out nomirror offset 0,0.5 font \"Helvetica,14\"\n",ticx[1],px[1],ticx[2],px[2],ticx[3],px[3],ticx[4],px[4]);
  fprintf(command, "set x2tics (\"\" %f, \"\" %f, \"\" %f) out\n",px1[1],px1[2],px1[3]);
  fprintf(command, "set ytics (\"%s\" %f, \"%s\" %f, \"%s\" %f, \"%s\" %f) out mirror rotate by 90 center offset -0.5,0 font \"Helvetica,14\"\n",ticy[1],py[1],ticy[2],py[2],ticy[3],py[3],ticy[4],py[4]);
  fprintf(command, "set cblabel \"Brightness [cnt s^{-1} deg^{-2}]\"offset 0.2,0  font \"Helvetica,14\"\n");
  fprintf(command, "set cbtics (\"1\" 1, \"10\" 10, \"10^{2}\" 100, \"10^3\" 1000) offset -0.7,0 rotate by 90 font \"Helvetica,14\"\n");
  fprintf(command, "set logscale cb\n");
  fprintf(command, "set cntrparam levels discrete %f+%f*5.,%f+%f*10.,%f+%f*20.,%f+%f*40.,%f+%f*80.,%f+%f*160.,%f+%f*320.,%f+%f*640.,%f+%f*1280.,%f+%f*2560.,%f+%f*5120.,%f+%f*10240.,%f+%f*20480.\n",*mean,*rms,*mean,*rms,*mean,*rms,*mean,*rms,*mean,*rms,*mean,*rms,*mean,*rms,*mean,*rms,*mean,*rms,*mean,*rms,*mean,*rms,*mean,*rms,*mean,*rms);
  fprintf(command, "set table '%s'\n",filename3);
  fprintf(command, "splot '%s'\n",filename2);
  fprintf(command, "unset table\n");
  fprintf(command, "set cntrparam levels discrete %f\n",*fl_border);
  fprintf(command, "set table '%s'\n",filename4);
  fprintf(command, "splot '%s'\n",filename2);
  fprintf(command, "unset table\n");
  
  fprintf(command, "set palette rgbformulae 22,13,-31\n");
  fprintf(command, "plot '%s' w image notitle, '%s' w lines lw 3 lc rgbcolor \"white\" notitle, '%s' w lines lw 1 lc rgbcolor \"black\" notitle\n",filename2,filename3,filename3);
  fprintf(command, "set output\n");
  fflush(command);
  fclose(command);
  }

////       fitting   the   image   with   2D-beta    model      /////  
void imagefit(char name[], int N_map, float x_map[], float y_map[], float fl_map[], float *maxfl, float *minfl, float *r_map, float cntx0, float cnty0, float *mean, float *rms, float coeff[], float *chi, float fl_mod[], float *fl_border, float *fit_peak_x0, float *fit_peak_y0, float *kappa)
{
  void mrqmin2D1(float x1[], float x2[], float y[], int ndata, float a[], int ia[], int ma, float **covar, float **alpha, float *chisq, void funcs2D(float, float, float [], float *, float [], int), float *alamda);//larger weight for cluster outer, for boundary fitting;

  void mrqmin2D2(float x1[], float x2[], float y[], int ndata, float a[], int ia[], int ma, float **covar, float **alpha, float *chisq, void funcs2D(float, float, float [], float *, float [], int), float *alamda);//larger weight for cluster center, for profile fitting;

  printf("\n......fitting image......\n");
  FILE *fp;
  float *a,**covar,**alpha,chisq[1],alamda[1];
  int i,j,*ia,ma,na;
  ma=8; na=ma;
  a=vector(1,ma); ia=ivector(1,ma);
  covar=matrix(1,ma,1,ma); alpha=matrix(1,ma,1,ma);

  //initialize the 8 fitting-coefficients
  a[1]=*maxfl; a[2]=*r_map/12.; a[3]=0.8; a[4]=0.1;//try profile slope \beta=0.8
  a[5]=0; a[6]=cntx0; a[7]=cnty0; a[8]=0.0;
  //printf("%f %f %f %f %f %f %f %f\n",a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8]);

  for(i=1;i<=ma;i++) //all of the 8 fitting-coefficients are free
    {ia[i]=1;}

  float chisq_tmp=0.,temp;
  int loop=0;
  
  for(i=1;i<=150;i++)
    {
      loop++;
      *alamda=-5.;
      mrqmin2D2(x_map,y_map,fl_map,N_map,a,ia,ma,covar,alpha,chisq,funcs2D,alamda);
      if(*chisq<chisq_tmp*0.99 || loop<=20)//the chi-square should be smaller and smaller
	{
	  //printf("%.2f %.2f %.2f %.2f\n",chisq_tmp,a[3],a[4],a[8]);
	  chisq_tmp=*chisq;
	}
      else if(a[3]>0.5 && a[4]>0.05 && a[4]<0.6)
	{
	  //the chi-square is stable and coefficients in reasonlabe ranges, adopted
	  break;
	}
      else
	{
	  chisq_tmp=chisq_tmp*1.2;//re-fitting with below new-initialization
	}
      if(a[3]<=0.5) {a[3]=0.8;}
      if(a[4]<=0.05) {a[4]=0.2;}
      if(a[4]>=0.6) {a[4]=0.4;}
    }
  temp=*chisq;

  for(i=1;i<=ma;i++)
    {
      coeff[i]=a[i];
    }

  //fit again, for new initialization
  a[1]=*maxfl; a[2]=*r_map/12.; a[3]=1.2; a[4]=0.1; //try profile slope \beta=1.2
  a[5]=0; a[6]=cntx0; a[7]=cnty0; a[8]=0.0;

  for(i=1;i<=ma;i++)//all of the 8 fitting-coefficients are free
    {ia[i]=1;}

  chisq_tmp=0.; loop=0;
    
  for(i=1;i<=150;i++)
    {
      loop++;
      *alamda=-5.;
      mrqmin2D2(x_map,y_map,fl_map,N_map,a,ia,ma,covar,alpha,chisq,funcs2D,alamda);
      if(*chisq<chisq_tmp*0.99 || loop<=20)//the chi-square should be smaller and smaller
	{
	  //printf("%.2f %.2f %.2f %.2f\n",chisq_tmp,a[3],a[4],a[8]);
	  chisq_tmp=*chisq;
	}
      else if(a[3]>0.5 && a[4]>0.05 && a[4]<0.6)
	{
	  //the chi-square is stable and coefficients in reasonlabe ranges, adopted
	  break;
	}
      else
	{
	  chisq_tmp=chisq_tmp*1.2;//re-fitting with below new-initialization
	}
      if(a[3]<=0.5) {a[3]=0.8;}
      if(a[4]<=0.05) {a[4]=0.2;}
      if(a[4]>=0.6) {a[4]=0.4;}
    }

  if(*chisq<temp)//adopte the new fitting-psizeults for smaller chi-square
    {
      for(i=1;i<=ma;i++)
	{
	  coeff[i]=a[i];
	}
    }

  //get the Kappa and peak position
  *kappa=(1.+coeff[4])/coeff[3];
  *fit_peak_x0=coeff[6];
  *fit_peak_y0=coeff[7];

  //refit with weight for outer region
  a[1]=*maxfl; a[2]=*r_map/12.; a[3]=0.8; a[4]=0.1;//try profile slope \beta=0.8
  a[5]=0; a[6]=cntx0; a[7]=cnty0; a[8]=0.5;
  //printf("%f %f %f %f %f %f %f %f\n",a[1],a[2],a[3],a[4],a[5],a[6],a[7],a[8]);

  for(i=1;i<=ma;i++) //all of the 8 fitting-coefficients are free
    {ia[i]=1;}

  chisq_tmp=0.; loop=0;

  for(i=1;i<=150;i++)
    {
      loop++;
      *alamda=-5.;
      mrqmin2D1(x_map,y_map,fl_map,N_map,a,ia,ma,covar,alpha,chisq,funcs2D,alamda);
      if(*chisq<chisq_tmp*0.99 || loop<=20)//the chi-square should be smaller and smaller
	{
	  //printf("%.2f %.2f %.2f %.2f\n",chisq_tmp,a[3],a[4],a[8]);
	  chisq_tmp=*chisq;
	}
      else if(a[3]>0.5 && a[4]>0.05 && a[4]<0.6)
	{
	  //the chi-square is stable and coefficients in reasonlabe ranges, adopted
	  break;
	}
      else
	{
	  chisq_tmp=chisq_tmp*1.2;//re-fitting with below new-initialization
	}
      if(a[3]<=0.5) {a[3]=0.8;}
      if(a[4]<=0.05) {a[4]=0.2;}
      if(a[4]>=0.6) {a[4]=0.4;}
    }
  temp=*chisq;

  for(i=1;i<=ma;i++)
    {
      coeff[i]=a[i];
    }

  //fit again, for new initialization
  a[1]=*maxfl; a[2]=*r_map/12.; a[3]=1.2; a[4]=0.1; //try profile slope \beta=1.2
  a[5]=0; a[6]=cntx0; a[7]=cnty0; a[8]=0.0;

  for(i=1;i<=ma;i++)//all of the 8 fitting-coefficients are free
    {ia[i]=1;}

  chisq_tmp=0.; loop=0;
    
  for(i=1;i<=150;i++)
    {
      loop++;
      *alamda=-5.;
      mrqmin2D1(x_map,y_map,fl_map,N_map,a,ia,ma,covar,alpha,chisq,funcs2D,alamda);
      if(*chisq<chisq_tmp*0.99 || loop<=20)//the chi-square should be smaller and smaller
	{
	  //printf("%.2f %.2f %.2f %.2f\n",chisq_tmp,a[3],a[4],a[8]);
	  chisq_tmp=*chisq;
	}
      else if(a[3]>0.5 && a[4]>0.05 && a[4]<0.6)
	{
	  //the chi-square is stable and coefficients in reasonlabe ranges, adopted
	  break;
	}
      else
	{
	  chisq_tmp=chisq_tmp*1.2;//re-fitting with below new-initialization
	}
      if(a[3]<=0.5) {a[3]=0.8;}
      if(a[4]<=0.05) {a[4]=0.2;}
      if(a[4]>=0.6) {a[4]=0.4;}
    }

  if(*chisq<temp)//adopte the new fitting-psizeults for smaller chi-square
    {
      for(i=1;i<=ma;i++)
	{
	  coeff[i]=a[i];
	}
    }

  //save the observation and the fitting-model in .dat file 
  char filename[30];
  sprintf(filename,"%s_obj.dat",name);

  fp=fopen(filename,"w+");
  fprintf(fp,"%4.0f %4.0f %8.2f\n",x_map[1],y_map[1],*minfl);
  for(i=2;i<=N_map;i++)
    {
      if(x_map[i]-x_map[i-1]>=1)
	{fprintf(fp,"\n");}
      fprintf(fp,"%4.0f %4.0f %8.2f\n",x_map[i],y_map[i],fl_map[i]);
    }
  fclose(fp);

  float *tx,*ty,*tr,r_mod,cnt_tot,cnt_sig;
  
  tx=vector(1,N_map); ty=vector(1,N_map); 
  tr=vector(1,N_map); 
  sprintf(filename,"%s_mod.dat",name);

  fp=fopen(filename,"w+");
  fprintf(fp,"%4.0f %4.0f %8.2f\n",x_map[1],y_map[1],*minfl);
  tx[1]=(x_map[1]-coeff[6])*cos(coeff[5])+(y_map[1]-coeff[7])*sin(coeff[5]);
  ty[1]=(y_map[1]-coeff[7])*cos(coeff[5])-(x_map[1]-coeff[6])*sin(coeff[5]);
  tr[1]=sqrt(pow(tx[1],2.)*pow(1.-coeff[4],2.)+pow(ty[1],2.))/(1.-coeff[4]);
  for(i=2;i<=N_map;i++)
    {
      tx[i]=(x_map[i]-coeff[6])*cos(coeff[5])+(y_map[i]-coeff[7])*sin(coeff[5]);
      ty[i]=(y_map[i]-coeff[7])*cos(coeff[5])-(x_map[i]-coeff[6])*sin(coeff[5]);
      tr[i]=sqrt(pow(tx[i],2.)*pow(1.-coeff[4],2.)+pow(ty[i],2.))/(1.-coeff[4]);
      if(x_map[i]-x_map[i-1]>=1)
	{fprintf(fp,"\n");}
      fl_mod[i]=coeff[1]*pow(1.+pow(tr[i]/coeff[2],2.),-1.*coeff[3])+coeff[8];
      fprintf(fp,"%4.0f %4.0f %8.2f\n",x_map[i],y_map[i],fl_mod[i]);
    }
  fclose(fp);

  //get the cluster boundary based on their actual size and shape
  int r_flag=1;
  for(i=1;i<10000;i++)
    {
      r_flag=r_flag+i;
      cnt_tot=0.;
      cnt_sig=0.;
      for(j=1;j<=N_map;j++)
	{
	  if(tr[j]>r_flag-1 && tr[j]<=r_flag)
	    {
	      cnt_tot=cnt_tot+1.;
	      if(fl_map[j]>*mean+5.*(*rms))
		{cnt_sig=cnt_sig+1.;}
	    }
	}
      if(cnt_sig<0.25*cnt_tot)
	{
	  *fl_border=coeff[1]*pow(1.+pow(r_flag/coeff[2],2.),-1.*coeff[3])+coeff[8];
	  break;
	}
    }
}

/////////////////////////read fits file////////////////////////

void readfits(char fitsname[], float psize, float cntx0, float cnty0, float *cntra, float *cntdec, int *nrow, int *ncol, float posx[], float posy[], float flux[], float flx[])
{
  int status=0;
  fitsfile *fptr;

  //open file and get rows & columns
  int nrows, ncols;
  float point_ra, point_dec, point_x0, point_y0;
  fits_open_file(&fptr, fitsname, READONLY, &status);
  fits_movabs_hdu(fptr, 1, NULL, &status);
  fits_read_key(fptr,TINT,"NAXIS1", &nrows, NULL, &status);
  fits_read_key(fptr,TINT,"NAXIS2", &ncols, NULL, &status);
  fits_read_key(fptr,TFLOAT,"CRVAL1", &point_ra, NULL, &status);
  fits_read_key(fptr,TFLOAT,"CRVAL2", &point_dec, NULL, &status);
  fits_read_key(fptr,TFLOAT,"CRPIX1", &point_x0, NULL, &status);
  fits_read_key(fptr,TFLOAT,"CRPIX2", &point_y0, NULL, &status);
  *nrow=nrows; *ncol=ncols;

  //read data as 1-D array
  int anynul,i,j,k=0,*nulval;
  long int nelements;
  long fpixel=1;

  nulval=ivector(1,1); 
  nelements=nrows*ncols; status=0; nulval[1]=1;
 
  fits_read_img(fptr,TFLOAT,fpixel,nelements,&nulval,flx+1, &anynul, &status);

  for(i=1;i<=nrows;i++)
    {
      for(j=1;j<=ncols;j++)
	{
	  k++;
	  posx[k]=i;
	  posy[k]=j;
	  flux[k]=flx[(j-1)*nrows+i];
	}
    }
  fits_close_file(fptr, &status);

  //calculate the ra, dec for cluster center
  *cntdec=point_dec+(cnty0-point_y0)*psize/3600.;  //in degree
  *cntra=point_ra-(cntx0-point_x0)*psize/(3600.*cos(3.14159*(*cntdec)/180.));  //in degree
  if(*cntra<0.)
    {*cntra=*cntra+360.;}
  if(*cntra>360.)
    {*cntra=*cntra-360.;}

}

////////calculate the mean flux & rms of backgrond region////////////
void backgrnd(int ndata, float x[], float y[], float fl[], float bgx0, float bgy0, float *mean, float *rms)
{
  int i,sum_pxl;
  float *fl_bg,sumfl; 
  fl_bg=vector(1,ndata);

  sum_pxl=0; sumfl=0.; *rms=0.;
  for(i=1;i<=ndata;i++)
    {
      if(sqrt(pow(x[i]-bgx0,2.)+pow(y[i]-bgy0,2.))<30) //radius of backgrnd region is set 30 pixls
	{
	  sum_pxl++;
	  fl_bg[sum_pxl]=fl[i];
	  sumfl=sumfl+fl_bg[sum_pxl];
	}
    }
  *mean=sumfl/(1.*sum_pxl);

  for(i=1;i<=sum_pxl;i++)
    {
      *rms=*rms+pow(fl_bg[i]-*mean,2.);
    }
  *rms=sqrt(*rms/(1.*sum_pxl));
}

//////////    select the region according to the cluster size  ////////
//////////          and save data in the region                ////////
void maprgn(int ndata, float x[], float y[], float fl[], float cntx0, float cnty0, float scale500, float z, float *mean, float *rms, float *r_map, float *minx, float *miny, float *maxx, float *maxy, float *minfl, float *maxfl, float *peak_x, float *peak_y)
{
  int i,annu_flag,max_flag,*count_tot,*count_sig;
  float *radi,*ratio,clstr_edge,rnan;
  
  annu_flag=0; max_flag=0;
  count_tot=ivector(1,2000); count_sig=ivector(1,2000);
  radi=vector(1,ndata); ratio=vector(1,2000);
  for(i=1;i<=2000;i++) //initialization
    {
      count_tot[i]=0; count_sig[i]=0; ratio[i]=0.; 
    }
  for(i=1;i<=ndata;i++)
    {
      radi[i]=sqrt(pow(x[i]-cntx0,2)+pow(y[i]-cnty0,2));
      annu_flag=floor(radi[i]/(scale500/50.))+1; //divide map in annuli with step of 10 kpc
      count_tot[annu_flag]++;
      if(fl[i]>*mean+5*(*rms))
	{count_sig[annu_flag]++;} //pixel counts with fluxes > 5sigma in each annulus
      if(max_flag<annu_flag)
	{max_flag=annu_flag;} //get the number of the largest annulus
    }
  rnan=max_flag*(scale500/50.); //physical size of the largest annulus

  if(z>5) //for clusters with no available redshift
    {
      for(i=1;i<=max_flag;i++)
	{
	  ratio[i]=(1.*count_sig[i])/(1.*count_tot[i]); //ratio of pixels with fluxes > 5sigma
	  if(ratio[i]<0.2) //the boundary is set when the 5sigma ratio < 20%
	    {
	      if(i<=45)
		{
		  clstr_edge=45*(scale500/50.);//diameter = 1 arcmin when angular size is very small 
		}
	      else
		{
		  clstr_edge=i*(scale500/50.);//actual size for clusters with larger size
		}
	      break;
	    }
	  if(i==max_flag)//for clusters partly out of the CCD coverage
	    {
	      if(i<=45)
		{
		  clstr_edge=45*(scale500/50.);
		}
	      else
		{
		  clstr_edge=max_flag;
		}
   	    }
	}
    }
  else  if(z>0.03)// for ordinary clusters with measured redshift
    {
      for(i=1;i<=max_flag;i++)
	{
	  ratio[i]=(1.*count_sig[i])/(1.*count_tot[i]);
	  if(ratio[i]<0.2)
	    {
	      if(i<=30)
		{
		  clstr_edge=30*(scale500/50.);//diameter = 1 Mpc when physical size is very small
		}
	      else
		{
		  clstr_edge=i*(scale500/50.);//actual diameter for larger cluster
		}
	      break;
	    }
	  if(i==max_flag)//for clusters partly out of the CCD coverage
	    {
	      if(i<=30)
		{
		  clstr_edge=40*(scale500/50.);
		}
	      else
		{
		  clstr_edge=max_flag;
		}
	    }
	}
    }
  else
    {
      for(i=1;i<=max_flag;i++)
	{
	  ratio[i]=(1.*count_sig[i])/(1.*count_tot[i]);
	  if(ratio[i]<0.2)
	    {
	      if(i<=20)
		{
		  clstr_edge=15*(scale500/50.);//diameter = 400 kpc when physical size is very small
		}
	      else
		{
		  clstr_edge=i*(scale500/50.);//actual diameter for larger cluster
		}
	      break;
	    }
	  if(i==max_flag)//for clusters partly out of the CCD coverage
	    {
	      if(i<=15)
		{
		  clstr_edge=15*(scale500/50.);
		}
	      else
		{
		  clstr_edge=max_flag;
		}
	    }
	}
    }
    
  *r_map=1.5*clstr_edge;
  
  *minx=10000.; *maxx=0.; *miny=10000.; *maxy=0.; *minfl=10000.; *maxfl=0.;

  for(i=1;i<=ndata;i++) //get the image edge and flux extremum
    {
      if(radi[i]<*r_map)
	{
	  if(x[i]<*minx) {*minx=x[i];}
	  if(x[i]>*maxx) {*maxx=x[i];}
	  if(y[i]<*miny) {*miny=y[i];}
	  if(y[i]>*maxy) {*maxy=y[i];}
	  if(fl[i]<*minfl) {*minfl=fl[i];}
	  if(fl[i]>*maxfl) {*maxfl=fl[i]; *peak_x=x[i]; *peak_y=y[i];}
	}
    }
}

////////////      redshift --> physical scale    /////////////
float integral(float z)
{
  float x,intestep,intemin,integral1,suminte,dist,om,ol,h;
  int i;
  h=0.7;
  om=0.3;
  ol=0.7;
  i=0;
  intemin=0.;
  integral1=0.;
  suminte=0.;
  intestep=z/1000;
  for (i=1;i<=1000;i++)
    {
      integral1=1/sqrt ((om*pow (1+i*intestep,3))+ol);
      suminte=suminte+integral1*intestep;
    }
  dist=suminte*3000./(h*(1+z));
  return dist;
}
